# AnImAl
#
#
# R function wrappers for the arithmetic subset of
# aniOp, implementing the history mechanism for
# animage objects
#

ia.add <- function (img1, img2, filter = NULL) {

  res <- aniOp(img1, "+", img2, filter)

  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }

  res

}

ia.sub <- function (img1, img2, filter = NULL) {

  res <- aniOp(img1, "-", img2)

  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }

  res
  
}

ia.mult <- function (img1, img2, filter = NULL) {

  res <- aniOp(img1, "*", img2)

  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }

  res
  
}

ia.div <- function (img1, img2, filter = NULL) {

  res <- aniOp(img1, "/", img2)

  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }

  res
  
}

# Positive leftshift means <<
# negative leftshift means >>
#
ia.shift <- function (img, leftshift, filter = NULL) {

  if(leftshift > 0)
    res <- aniOp(img, "<<",  as.integer(leftshift), filter)
  else
    res <- aniOp(img, ">>", -as.integer(leftshift), filter)

  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }

 res
  
}

#
# This is a wrapper that maps vectorized
# R functions acting on a variable number of arguments
# and producing an integer or real scalar.
# The function operates separately on image@data
# and image@outside returning an image of the same type
# returned by 'op'.
#
# a <- ia.op(`+`, sampleFace, 1)
# a <- ia.op(`+`, sampleFace, sampleFace)
#
ia.matop <- function(matop, ...) {
  #
  document <- TRUE
  focus    <- NULL
  args     <- list(...)
  intImgs  <- 0
  realImgs <- 0

  for(a in args) {
    if(is.animage(a)) {
      document <- document && a@document

      if(a@storage == "real")
        realImgs <- realImgs + 1
      else
        intImgs  <- intImgs + 1
      
      if(is.null(focus))
        focus <- a@focus
      else if(sum(abs(a@focus - focus)) > 0)
        stop("image foci differs")
    }
  }

  #
  # a '@data' extractor and document
  # request tracker
  #
  f.roi <- function(x) {
    if(is.animage(x))
      x@data
    else
      x
  }
  #
  # a '@outside' tracker
  #
  f.out <- function(x) {
    if(is.animage(x))
      x@outside
    else
      x
  }
  #
  # generate the matrix result
  #
  res.roi <- do.call(matop, 
                     ia.map(f.roi,
                            args))
  #
  # and the 'outside' result
  #
  res.out <- do.call(matop, 
                     ia.map(f.out,
                            args))

  #
  # pack them into the final result
  #
  if(realImgs == 0) {
    res      <- animage(data     = array(as.integer(res.roi), dim(res.roi)),
                        pmode    = "scalar",
                        outside  = as.integer(res.out[1]),
                        storage  = "integer",
                        document = document,
                        focus    = focus)
    
  } else {
    res      <- animage(data     = res.roi,
                        pmode    = "scalar",
                        outside  = as.real(res.out[1]),
                        storage  = "real",
                        document = document,
                        focus    = focus)
  }
  #
  res
}

#
# The following functions are simple wrappers based on
# R vectorized functions
#
ia.abs <- function(img1) {

  res <- ia.matop(abs, img1)

  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }
  
  res
}

ia.sqrt <- function(img1) {

  res <- ia.matop(sqrt, img1)

  
  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }
  
  res
}

ia.atan2 <- function(img1, img2) {
  
  res <- ia.matop(atan2, img1, img2)
  
  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }
  
  res
}

