#
#
#
aniOp <- function(img1,
                  op,
                  img2,
                  filter = NULL)
{
  if(!(is.animage(img1)))
    stop("first argument must be an animage")

  if(!(is.character(op)))
    stop("second argument must be an operation defining string")

  if(is.animage(img2))
                                        # perform on image
    {
      if((img1@storage == "integer") && (img2@storage == "integer")) {
        aniOpPrefix <- "aniOpInt";
      } else if ((img1@storage == "real") && (img2@storage == "real")) {
        aniOpPrefix <- "aniOpReal";
      } else {
        stop("mismatch of animage/animage STORAGEs")
      }
      
      if(is.null(filter))
        {
                                        # we compute the mask
                                        # and hand over to aniOp_im_im_mk
          x1  <- img1@focus[1]
          y1  <- img1@focus[2]
          dx1 <- dim(img1@data)[2]
          dy1 <- dim(img1@data)[1]

          x2  <- img2@focus[1]
          y2  <- img2@focus[2]
          dx2 <- dim(img2@data)[2]
          dy2 <- dim(img2@data)[1]

          xf  <- min(x1, x2)
          yf  <- min(y1, y2)
          dxf <- max(x1+dx1, x2+dx2) - xf
          dyf <- max(y1+dy1, y2+dy2) - yf

          .Call(paste(aniOpPrefix, "im_im_mk", sep = "_"), 
                img1, op, img2, animask(xf, yf, dxf, dyf),
                PACKAGE="AnImAl")
        }
      else if (is.animask(filter))
        {
          .Call(paste(aniOpPrefix, "im_im_mk", sep = "_"), 
                img1, op, img2, filter,
                PACKAGE="AnImAl")
        }
      else if (is.animage(filter))
        {
          .Call(paste(aniOpPrefix, "im_im_im", sep = "_"), 
                img1, op, img2, filter,
                PACKAGE="AnImAl")
        }
    }
  else if (is.numeric(img2))
                                        # perform 'on' constant
    {
      if((img1@storage == "integer") && is.integer(img2)) {
        .Call("aniOpInt_im_ss",
              img1, op, img2,
              PACKAGE="AnImAl")

      } else if ((img1@storage == "real") && is.real(img2)) {
        .Call("aniOpReal_im_ss",
              img1, op, img2,
              PACKAGE="AnImAl")
      } else {
        stop("mismatch of animage/scalar STORAGEs")
      }

    }
}

#
# Get image based on mask, filter, or coordinates
#
aniGet <- function(img,
                   locus)
{
  if(!(is.animage(img)))
    stop("first argument must be an animage")

  if(img@storage == "integer") {
    aniOpPrefix <- "aniGetInt";
  } else if (img@storage == "real") {
    aniOpPrefix <- "aniGetReal";
  }

  if(is.animask(locus)) {
    .Call(paste(aniOpPrefix, "mk", sep = "_"),
          img, locus,
          PACKAGE="AnImAl")
  }
  else if(is.animage(locus)) {
    .Call(paste(aniOpPrefix, "im", sep = "_"),
          img, locus,
          PACKAGE="AnImAl")
  }
  else if((typeof(locus) == "integer") && length(locus)  == 2) {
    .Call(paste(aniOpPrefix, "px", sep = "_"),
          img, locus,
          PACKAGE="AnImAl")
  } else {
    stop("illegal LOCUS specification")
  }
}

aniSet <- function(img,
                   locus,
                   value)
{
  if(!(is.animage(img)))
    stop("first argument must be an animage")

  if((typeof(locus) == "integer") && length(locus)  == 2) {
    if(img@storage == "integer") {
      .Call("aniSetInt_px",
            img,
            locus,
            as.integer(value),
            PACKAGE="AnImAl")
    } else if (img@storage == "real") {
      .Call("aniSetReal_px",
            img,
            locus,
            as.real(value),
            PACKAGE="AnImAl")
    }
  } else {
    stop("illegal LOCUS specification")
  }
}
