#
#
#
ia.gaussianSmooth <- function(img,
                              sigma = 1.0)
{
  if(!(is.animage(img)) || !(is.real(sigma)))
    stop("first argument must be an animage, second argument must be a real value")
  
  if(img@storage == "real") {
    res <- .Call("aniGaussianR",
                 img,
                 sigma,
                 PACKAGE="AnImAl")
  } else if(img@storage == "integer") {
    res <- .Call("aniGaussianI",
                 img,
                 sigma,
                 PACKAGE="AnImAl")
  } else {
    stop("unsupported image type")
  }

  res@outside <- img@outside
  
  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }

  res
}
#
#
#
ia.zeroCrossings <- function(img,
                             sigma = 1.0)
{
  if(!(is.animage(img)) || !(is.real(sigma)))
    stop("first argument must be an animage, second argument must be a real value")

  document     <- img@document
  img@document <- FALSE

  if(img@storage != "real")
    img <- ia.realAnimage(img)
  
  res <- ia.edge(ia.integerAnimage(ia.greater(ia.sub(ia.gaussianSmooth(img, sigma),
                                                     ia.gaussianSmooth(img, 1.6*sigma)),
                                              0.0)))

  img@document <- document
  
  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }

  res
}
