#
#
#
ia.gradient <- function(img,
                        Kx = array(c(-1,0,1), c(1,3)),
                        Ky,
                        smoothing = 1.0,
                        metrics   = 1,
                        direction = FALSE,
                        thr       = 1e-15) {
  #
  document     <- img@document
  #
  if(img@storage == "integer")
    img <- ia.realAnimage(img)
  #
  img@document <- FALSE
  #
  # kernels used to compute gradient by convolution
  # are usually related by a 90 degrees rotation
  #
  if(missing(Ky))
    Ky = t(Kx)
  #
  if(smoothing > 0)
    img <- ia.gaussianSmooth(img, sigma = smoothing)
  #
  # compute the derivatives,
  #
  Dx         <- ia.convolution(img, Kx)
  Dy         <- ia.convolution(img, Ky)
  #
  Dx@outside <- 0
  Dy@outside <- 0
  #
  # the resulting magnitude,
  #
  if(metrics == 1) {
    M   <- ia.add(ia.abs(Dx), ia.abs(Dy))
  } else if (metrics == 2) {
    M   <- ia.sqrt(ia.add(ia.mult(Dx,Dx), ia.mult(Dy,Dy)))
  } else {
    stop("illegal gradient magnitude specification")
  }
  #
  # and gradient direction (if required)
  #
  if(direction == TRUE) {
    D <- ia.mult(ia.greater(M, thr),
                 ia.mult(ia.atan2(Dy, Dx), 180.0 / pi))
  }
  #
  img@document <- document
  #
  if(direction)
    res          <- list(magnitude=M,
                         direction=D)
  else
    res <- M
  #
}


