#
#
#
ia.table <- function(img,
                     table,
                     tableOffset = 0L)
{
  if(!(is.animage(img)))
    stop("first argument must be an animage")

  if(!(is.integer(table)))
    stop("second argument must be an integer vector")

  if(!(is.integer(tableOffset) && (length(tableOffset) == 1)))
    stop("tableOffset must be a scalar integer")

  res <- .Call("aniTable",
               img,
               table,
               tableOffset,
               PACKAGE="AnImAl")

  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }

  res
}

#
# This is an R level implementation of the same function used to assess
# the relative speed of C vs R code (it turns out that C is ~40x faster)
#
ia.tableR<- function(img,
                     table,
                     tableOffset = 0L)
{
  if(!(is.animage(img)))
    stop("first argument must be an animage")

  if(!(is.integer(table)))
    stop("second argument must be an integer vector")

  if(!(is.integer(tableOffset) && (length(tableOffset) == 1)))
    stop("tableOffset must be a scalar integer")

  data1 <- img@data
  datar <- array(as.integer(0), dim(data1))

  i1 <- length(table)

  for(y in seq(1, dim(data1)[1])) {
    for(x in seq(1, dim(data1)[2])) {
      value <- data1[y,x] - tableOffset + 1

      if((value < 1) || (value > i1))
        datar[y,x] <- data1[y,x]
      else 
        datar[y,x] <- table[value]
    }
  }

  res <- animage(datar,
                 "scalar",
                 focus    = img@focus,
                 outside  = table[max(min(i1, img@outside - tableOffset + 1), 1)],
                 document = img@document)
  
  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }

  res
}

# b <- aniTableR(animage(array(0, c(256,256))), as.integer(1:256, 4L))
