\name{aniIO}

\alias{aniIO}
\alias{tiffstat}
\alias{readtiff}
\alias{readpng}
\alias{readimage}
\alias{writetiff}
\alias{ia.readAnimageChannel}
\alias{ia.readAnimage}
\alias{ia.stretchRange}
\alias{ia.mapRange}
\alias{ia.clipRange}
\alias{ia.averageImageChannels}


\title{Basic image reading and writing support}

\description{

   Read an RGB  TIFF (or PNG) image with alpha  channel from the given
file returning  a list of four  channels each represented  by a single
\code{animage}.

}

\usage{
readpng(filename)
readtiff(filename)
readimage(filename)
tiffstat(filename)
writetiff(imageChannels, filename, dataFormat = 0, hasTransparency =
FALSE)

ia.readAnimage(filename, document = FALSE)
ia.readAnimageChannel(info, channel, document = FALSE)

ia.averageImageChannels(img, channels, weights)

ia.clipRange(img, clipRange)
ia.mapRange(img, srcRange, dstRange)
ia.stretchRange(img, dstRange)
}

\arguments{
  \item{channel}{an integer number from 1 to 4 identifying the required
	channel}
  \item{channels}{an integer vector specifying which channels should be
	used for the creation of the monochrome channel: its dimension
	should be the same as that of \code{weights}}
  \item{dataFormat}{an integer number: 0 for 8 bits, 1 for 16 bits, and
	2 for float values}
  \item{document}{a logical specifying whether self documenting AnImAl
	functionalities should be activated}
  \item{filename}{a string identifying the file name (including the .tif
	suffix)}
  \item{hasTransparency}{a boolean:  TRUE if the last  channel should be
considered as an alpha channel}
  \item{imageChannels}{a list of integer \code{animage}s}
  \item{img}{an \code{animage} object}
  \item{srcRange}{a two value vector \code{c(low,high)}}
  \item{dstRange}{a two value vector  \code{c(low,high)}}
  \item{info}{a string identifying the file name (including the .tif
	suffix) or a list of arrays returned by \code{readtiff} or \code{readpng}}
  \item{weights}{a real vector specifying the weights used for the
	linear combination of the channels representing the monochrome image}
}

\value{

  Function \code{tiffstat} return a list of 6 integer values
  representing  the values of  the following  tags: \code{SAMPLEFORMAT},
\code{SAMPLESPERPIXEL},    \code{BITSPERSAMPLE},    \code{EXTRASAMPLES},
\code{IMAGELENGTH}, and \code{IMAGEWIDTH}. Functions \code{readtiff} and
\code{readpng} return a list of (typically four) arrays representing

	\item{1_comp}{the first image channel  (red for integer TIFF/PNG images)}
	\item{2_comp}{the second image channel (green for integer TIFF/PNG images)}
	\item{3_comp}{the third image channel  (blue for integer TIFF/PNG images)}
	\item{4_comp}{the fourth image channel (usually the transparency channel)}
	
	In the case of TIFF images the number of returned channels may vary
	depending on the image being color or greyscale: real arrays may be
	returned in the case of TIFF float format.
	
Function \code{ia.getAnimage} is an  interface to the previous functions
returning     a     list     of     \code{animage}     objects     while
\code{ia.getAnimageChannel} returns a single \code{animage} representing
the selected channel.

}

\details{

Pixels  are  usually represented  with  8  bits  per channel  and  the
resulting \code{animage}  objects are of type  \code{integer}. The low
level  input  functions  \code{readpng}  and \code{readtiff}  always
return a list of four channels, even in the case of grey level images.
The TIFF  format also supports  a pixel value representation  based on
IEEE floating  point numbers.  While the case  of single layer  PNG or
integer TIFF  images is  correctly handled (and  the result is  a four
channel  representation of  a grey  image) this  does  not (currently)
apply to floating point TIFF  images that must be natively stored with
three or  more channels  (the fourth  one is assumed  to be  the alpha
channel and any additional channel discarded).

If no alpha channel is present,  one is added (alpha values default to
255 for the 8bit case and to 1.0 for the floating point case). Function
\code{readimage} selects the \code{readtiff} or \code{readpng} function
based on the filename suffix.


}

\author{Roberto Brunelli \email{brunelli@fbk.eu}}

\seealso{
   \link{AnImAl}
}

\examples{
#
# get paths to package samples
#
sampleimages <- file.path(system.file(package = "AnImAl"), "sampleimages")
#
# Read a standard TIFF file with alpha channel
#
sampleFace <- readtiff(file.path(sampleimages, "sampleFace_06.tif"))
}
\keyword{array}
\keyword{file}
