\name{aniOperations}

\alias{aniOperations}
\alias{ia.add}
\alias{ia.div}
\alias{ia.mult}
\alias{ia.shift}
\alias{ia.sub}

\alias{ia.and}
\alias{ia.or}
\alias{ia.not}
\alias{ia.xor}

\alias{ia.equal}
\alias{ia.greater}
\alias{ia.greater_or_eq}
\alias{ia.less}
\alias{ia.less_or_eq}

\alias{ia.min}
\alias{ia.max}
\alias{ia.abs}
\alias{ia.sqrt}
\alias{ia.atan2}
\alias{ia.matop}

\title{AnImAl arithmetical, logical, relational operations}

\description{

   The  basic  set  of  AnImAl operations  working  on  \code{animage}
objects,  supporting  processing self  documentation  via the  history
mechanism.

}

\usage{
ia.add(img1, img2, filter = NULL)
ia.div(img1, img2, filter = NULL)
ia.mult(img1, img2, filter = NULL)
ia.shift(img, leftshift, filter = NULL)
ia.sub(img1, img2, filter = NULL)

ia.and(img1, img2, filter = NULL)
ia.not(img1)
ia.or(img1, img2, filter = NULL)
ia.xor(img1, img2, filter = NULL)

ia.equal(img1, img2, filter = NULL)
ia.greater(img1, img2, filter = NULL)
ia.greater_or_eq(img1, img2, filter = NULL)
ia.less(img1, img2, filter = NULL)
ia.less_or_eq(img1, img2, filter = NULL)

ia.max(img1, img2, filter = NULL)
ia.min(img1, img2, filter = NULL)

ia.abs(img1)
ia.sqrt(img1)
ia.atan2(img1, img2)
ia.matop(matop, ...)
}

\arguments{
  	\item{img,img1}{an \code{animage} object}
	\item{img2}{an \code{animage} object or a scalar numeric value matching the storage mode of \code{img1}}
	\item{filter}{an \code{animask} or an \code{animage} identifying the
	  region where computation will be performed}
	\item{leftshift}{an integer value: a negative value corresponds to a
	  rightshift (by an amount equal to the absolute value)}
	\item{...}{a variable number of lists to whose elements \code{op} is
	  applied}
	\item{matop}{a vectorized R operation acting on integers and reals
	  returning an integer or real value}
}

\value{
 A new \code{animage}.
}

\details{
	These  binary functions operate  on \code{animage}  objects taking
into account  the value of  the \code{outside} slot  of \code{animage}
objects and their history mechanisms. The represent the preferred user
and    developer   interface   to    the   functions    described   in
\link{aniOps}.

 Function \code{ia.matop} is a simple wrapper to R vectorized functions
 that are able to work directly on array data. The wrapper does not
 support region of interest filtering.
 
}

    
\author{Roberto Brunelli \email{brunelli@fbk.eu}}

\seealso{
   \link{aniOps}, \link{aniIO}, \code{\link[pixmap]{pixmap}}
}

\examples{
#
# get paths to package samples
#
sampleimages <- file.path(system.file(package = "AnImAl"), "sampleimages")
#
# Read the green channel of a TIFF picture
#
sampleFace <- ia.readAnimageChannel(file.path(sampleimages, "sampleFace_06.tif"), 2)
#
# create an animask for a region of interest 
#
sampleFaceDetailMask <- animask(66, 59, 92, 169)
#
# extract the detail
#
sampleFaceDetail <- ia.get(sampleFace, sampleFaceDetailMask)
#
# highlight the detail 
#
highlightedDetail <- ia.add(ia.div(sampleFace, 4L), sampleFaceDetail)
#
# and finally view it
#
ia.show(highlightedDetail)
#
# ia.matop provides an handy wrapper to extended AnImAl
# image operations, preserving self documenting capabilities
#
ia.show(ia.matop(`-`, 1, sampleFace))
}
\keyword{array}
