\name{aniUtilities}

\alias{aniUtilities}

\alias{ia.mask}
\alias{ia.maskLocator}
\alias{ia.frame}
\alias{ia.get}
\alias{ia.updatedHistory}
\alias{ia.up}
\alias{ia.down}
\alias{ia.left}
\alias{ia.right}
\alias{ia.slide}
\alias{ia.translateAt}
\alias{ia.congruent}
\alias{ia.scale}
\alias{ia.size}

\alias{ia.packBitImages}
\alias{ia.unpackBitImages}

\alias{ia.integerAnimage}
\alias{ia.realAnimage}
\alias{ia.apply}
\alias{ia.reduce}
\alias{ia.map}
\alias{ia.statImage}
\alias{ia.xmax}
\alias{ia.xmin}
\alias{ia.ymax}
\alias{ia.ymin}

\alias{ia.subsample2DMatrix}
\alias{ia.subsample}

\title{AnImAl support functions}

\description{

	A  small set  of utility  functions supporting  the coding  of novel
AnImAl  functions, translating and  casting \code{animage}  objects, and
parallel processing of binary images.

}

\usage{
ia.congruent(A, B = FALSE)

ia.translateAt(img, at = c(0L, 0L))
ia.down(img, delta = 1L)
ia.left(img, delta = 1L)
ia.right(img, delta = 1L)
ia.up(img, delta = 1L)
ia.slide(img, dx = 1L, dy = 1L)

ia.subsample(img, factor = 2)
ia.subsample2DMatrix(mat, factor = 2)

ia.get(img, locus)

ia.mask(img)
ia.maskLocator(img)

ia.statImage(img)
ia.xmin(img)
ia.xmax(img)
ia.ymin(img)
ia.ymax(img)

ia.integerAnimage(img)
ia.realAnimage(img)

ia.packBitImages(intAnimageList, thr = 128L)
ia.unpackBitImages(img, planes = 8L)

ia.scale(A, maxValue = 1.0)

ia.apply(img, f)
ia.reduce(op, X)
ia.map(op, ...)

ia.updatedHistory(callinfo)
}

\arguments{
  
	\item{img}{an \code{animage}}
	\item{callinfo}{an object of type \code{call}}
	\item{delta}{an integer representing a displacement}
	\item{dx}{an integer representing a displacement in the horizontal direction}
	\item{dy}{an integer representing a displacementin the vertical direction}
   	\item{locus}{an \code{animask}, an integer \code{animage}, or a vector of two integers \code{c(x,y)}}
   	\item{at}{a vector of two integers \code{c(x,y)}}
    \item{intAnimageList}{a list of integer \code{animage} objects}
    \item{A}{an animage or a list of  \code{animage} objects}
    \item{B}{an animage or a list of  \code{animage} objects}
	\item{thr}{an integer value used to binarize images}
	\item{planes}{the number of image planes within a packed image}
	\item{maxValue}{a real}
	\item{f}{a function f(x,y,z) of three arguments representing the
	  image coordinates (x,y) and the corresponding value }
	\item{X}{a list of (homogeneous) values}
	\item{mat}{a two dimensional matrix}
	\item{factor}{an integer or a list of two integers}
	\item{...}{a variable number of lists to whose elements \code{op} is applied}
	\item{op}{a binary function to be applied as \code{op(X[[1]],
		op(X[[2]], ...))}}
}

\value{
 
Functions \code{ia.mask} and \code{ia.maskLocator} return an \code{animask}.

Functions \code{ia.up}, \code{ia.down}, \code{ia.left}, \code{ia.right},
\code{ia.apply},      \code{ia.slide},      \code{ia.subsample}      and
\code{ia.packBitImages} return an \code{animage}.

Function \code{ia.scale} return a (list of) \code{animage}(s).

Function \code{ia.reduce} return value is of the type returned by \code{op}.

Function \code{ia.map} return value is  a list whose elements are of the
type returned by \code{op}.

Function \code{ia.get} returns  an \code{animage} when \code{locus} is
an \code{animask} or an \code{animage} and it returns a single numeric
value when \code{locus} is a vector.

Function \code{ia.updatedHistory} returns a logical \code{FALSE} or an
object of type \code{call}.

Function \code{ia.unpackBitImages}  returns a list of  objects of type
\code{animage}.

Function \code{ia.congruent} returns a logical value.

Function \code{ia.subsample2DMatrix} returns a two dimensional array.
}		

\details{

	Function  \code{ia.mask} returns  the  \code{animask} representing
the extent  of its  argument while function  \code{ia.get} is  used to
access the contents of an \code{animage} object. Function
\code{ia.maskLocator} returns a mask representing the bounding box of
two points selected by the user when \code{img} has been plotted with
the function \code{ia.show}. 

    Function \code{ia.show} plots the given image represented by a
	single \code{animage} or a list of three (R, G, B) \code{animages}.
	
    Function \code{ia.size} returns the area  of the \code{animage}
	region of interest.
	
    Functions   \code{ia.up},    \code{ia.down},   \code{ia.left},   and
\code{ia.right} are used to change  the values of \code{focus} within an
\code{animage} object:  the effect is that of  translating image content
in   the  corresponding   direction  (by   the   specified  \code{delta}
amount).  Function \code{ia.slide}  is similar  but acts  on  both focus
coordinates.

	The  function \code{ia.packBitImages}  is used  to store  within a
single integer  \code{animage} a set  of binary images.   The function
\code{ia.unpackBitImages}  is  used   to  extract  the  image  planes,
returning  them as  a list  of \code{animage}  objects (the  number of
planes may be specified).

    Functions \code{ia.integerAnimage}  and \code{ia.realAnimage} can be
considered as casting functions, simply changing the storage type of the
pixel values (with the usual castings among integer and real values).

   Function \code{ia.congruent} returns \code{TRUE} if all its arguments are
   images   whose  region   of  interest   coincide   (and  \code{FALSE}
   otherwise).

   Function \code{ia.scale} returns a list of \code{animage}s linearly
   scaled so that their maximum value matches the required one.

   Function \code{ia.subsample} reduces the resolution of the given
   image by subsampling the original image according to
   \code{factor}. If the latter is a single integer, both image
   dimensions are reduced by the same factor, otherwise the first
   element of factor determines the reduction of the number of rows and
   the second one that of columns. Image \code{focus} is reduced
   accordingly.    Function  \code{ia.subsample2DMatrix}   is   used  by
   \code{ia.subsample}  and  has a  similar  semantics targeting  matrices
instead of images.

   
   Function \code{ia.reduce} is not specific to \code{animage}s: it
   recursively applies a binary function \code{op}, computing
   \code{op(X[[1]], op(X[[2]], ...))}. Function \code{ia.map} is similar
  }

\author{Roberto Brunelli \email{brunelli@fbk.eu}}

\examples{
#
# Load required package
#
require(AnImAl)
#
# get paths to package samples
#
sampleimages <- file.path(system.file(package = "AnImAl"), "sampleimages")
#
# Read a binary image
#
charA <- ia.readAnimageChannel(file.path(sampleimages, "a.tif"), 1)
#
# create a sequence of traslated images
#
As <- c()
for(i in 1:10) {
 As <- c(As, ia.left(charA, delta = i*10))
}
#
# and pack them into a single image
#
packedChars <- ia.packBitImages(As)
#
# and view it 
#
ia.show(packedChars)
}
\keyword{array}

