\name{ia.edge}

\alias{ia.edge}

\title{A simple edge detector for binary images}

\description{

   Returns the 8-connected boundary of the objects in a binary image. It works also for packed binary images, performing the edge extraction in parallel for all planes: the result can then be unpacked into the single component images.

}

\usage{ia.edge(img)}

\arguments{
   	\item{img}{an \code{animage} that can be interpreted as a binary image or as a set of packed binary images}
}

\value{
 A binary \code{animage} (or a set of packed binary \code{animage}s).
}

\details{
The  function   computes  the  boundary  using  only   basic  logic  and
translation  AnImAl  operations.  A  similar  approach  can  be used  to
compute the 4-connected boundary.
}


\author{Roberto Brunelli \email{brunelli@fbk.eu}}

\seealso{
   \link{aniIO}, \link{aniOperations}, 
   \code{\link{ia.packBitImages}}, \code{\link{ia.unpackBitImages}}.
}

\examples{
#
# get paths to package samples
#
sampleimages <- file.path(system.file(package = "AnImAl"), "sampleimages")
#
# Read a few binary images
#
charA <- ia.readAnimageChannel(file.path(sampleimages, "a.tif"), 1)
charB <- ia.readAnimageChannel(file.path(sampleimages, "b.tif"), 1)
charC <- ia.readAnimageChannel(file.path(sampleimages, "c.tif"), 1)
#
# pack them into a single image
#
charABC <- ia.packBitImages(c(charA, charB, charC))
#
# perform edge extraction
#
charABCedge <- ia.edge(charABC)
#
# have a look
#
ia.show(ia.not(charABCedge))
#
# ... and unpack the results.
#
charABCedges <- ia.unpackBitImages(charABCedge)
charAedge    <- charABCedges[[1]]
charBedge    <- charABCedges[[2]]
charCedge    <- charABCedges[[3]]
#
}
\keyword{array}
