/*
 * (c) 1993,1994,1995,1996,1997,1998 by ITC-IRST
 *
 */

#ifndef AFFINE_HPP
#define AFFINE_HPP

#include <arrayTools/Array.h>

#include <arrayTools/namespace_header.h>


namespace _ARRAYTOOLS_NAMESPACE
{
  //Old and "bugged" versions
  //! Returned matrix values are forced in [0.0, 255.0] 
  array<u_char>&	Affine(array<u_char>& I, array<double>& M, vector<double> &O);
  array<float>&	        Affine(array<float>& I,  array<double>& M, vector<double> &O);
  array<double>&	Affine(array<double>& I, array<double>& M, vector<double> &O);
  

  //! Return the affine transform of I according to 
  //! matrix M and translation vector O
  //! O = M(I+O)
  template<class U, class T>
    array<U>& Affine(array<T>& I,
		     double a,
		     double b,
		     double c,
		     double d,
		     double o0 = 0.0,
		     double o1 = 0.0,
		     double unmapped_color = 0.0)
    {

      array<U> * O;
      U ** _O;
      T ** _I = I.storage();

      double	ia, ib, ic, id;
      double	bounds[4][2];
      double	determinant;
      int		cols, rows;
      int		x_start, x_end;
      int		y_start, y_end;
      double	x, y, x_min, x_max, y_min, y_max;
      bool		degenerate = false;
   
      cols		= I.cols();
      rows		= I.rows();
      int r0, r1, c0, c1;
      I.boundary(r0,r1,c0,c1);
      
      /* Compute determinant */
      determinant	= a*d - b*c;
   
      /* Check if transform is singular */
      if (determinant <= 0.000001)
	{
	  fprintf(stderr,"AffineTransform : Singular transform with det=%g\n",determinant);
	  exit(1);
	}

      /* Trap degenerate image */
      if ((rows == 1) && 
	  (cols == 1) && 
	  (a == d) && 
	  (determinant <= (1.0 + 0.000001)))
	degenerate = true;

      /* Compute inverse transform */
      ia	=   (d / determinant);
      ib	= - (b / determinant);
      ic	= - (c / determinant);
      id	=   (a / determinant);

      /* Get bounds of image */

      bounds[0][1]	= (double) c0;
      bounds[0][0]	= (double) r0;
      bounds[1][1]	= bounds[0][1] + (double)cols;
      bounds[1][0]	= bounds[0][0]; 
      bounds[2][1]	= bounds[0][1]; 
      bounds[2][0]	= bounds[0][0] + (double)rows;
      bounds[3][1]	= bounds[1][1];
      bounds[3][0]	= bounds[2][0];

      /* Compute new bounding box */
      x_min = x_max = a * (bounds[0][0]+o0) + b * (bounds[0][1]+o1);
      y_min = y_max = c * (bounds[0][0]+o0) + d * (bounds[0][1]+o1);

      {
	double	b_x, b_y;
	int	i;
      
	for(i = 1; i < 4; i++)
	  {
	    b_x  = a * (bounds[i][0]+o0) + b * (bounds[i][1]+o1);
	    b_y	 = c * (bounds[i][0]+o0) + d * (bounds[i][1]+o1);
	    x_min = MIN(x_min, b_x);
	    x_max = MAX(x_max, b_x);
	    y_min = MIN(y_min, b_y);
	    y_max = MAX(y_max, b_y);
	  }
      }
    
      x_start	= (int) (rint(x_min));
      x_end	= (int) (rint(x_max));
      y_start	= (int) (rint(y_min));
      y_end	= (int) (rint(y_max));
      
    
      if (degenerate)
	{
	  O = new array<U>(x_start,y_start,x_start+1,y_start+1);
	  _O = O->storage();
	  _O[x_start][y_start] = I[r0][c0];
	  return( O->temporary() );
	}

      O = new array<U>(x_start,x_end-1,y_start,y_end-1);

      _O = O->storage();
    
      /* Fill it */
      {
	double	back_x, back_y, t1, t2, t0;
	int     bx, by;
	double	v00, v01, v10, v11;
	int	xx, yy;
      
	for(x = x_min, xx = x_start; xx < x_end; x++, xx++)
	  {
	    for(y = y_min, yy = y_start; yy < y_end; y++, yy++)
	      {
		back_x = (ia * (x + 0.5) + ib * (y + 0.5)) - 0.5 - o0;
		back_y = (ic * (x + 0.5) + id * (y + 0.5)) - 0.5 - o1;
		bx     = (int)floor(back_x);
		by     = (int)floor(back_y);
	      
		v00 = v01 = v10 = v11 = 0;
	      
		/* Read surrounding values of back projection */
	      
		if ((r0 <= bx) && (bx <=r1))
		  {
		    if ((c0 <= by) && (by <= c1))
		      v00 = _I[bx][by];
		    if ((c0 <= by+1) && (by+1 <= c1))
		      v01 = _I[bx][by+1];
		  }
		if ((r0 <= bx+1) && (bx+1 <= r1))
		  {
		    if ((c0 <= by) && (by <= c1))
		      v10 = _I[bx+1][by];
		    if ((c0 <= by+1) && (by+1 <= c1))
		      v11 = _I[bx+1][by+1];
		  }

		t0  = (back_y - by);
		t1  = v00 + t0 * (v01 - v00);
		t2  = v10 + t0 * (v11 - v10);
	      
		_O[xx][yy] = (U)rint(t1+(back_x-bx)*(t2-t1));
	      }
	  }
      }
      return O->temporary();
    }

  //! Return the affine transform of I according to 
  //! matrix M and translation vector O
  //! O = M(I+O)
  template<class U, class T>
    array<U>& GeneralAffine(array<T>& I,
			    double a,
			    double b,
			    double c,
			    double d,
			    double o0 = 0.0,
			    double o1 = 0.0,
			    double unmapped_color = 0.0)
    {

      array<U> * O;
      U ** _O;
      T ** _I = I.storage();

      double	ia, ib, ic, id;
      double	bounds[4][2];
      double	determinant;
      int		cols, rows;
      int		x_start, x_end;
      int		y_start, y_end;
      double	x, y, x_min, x_max, y_min, y_max;
      bool		degenerate = false;
   

      cols		= I.cols();
      rows		= I.rows();
      int r0, r1, c0, c1;
      I.boundary(r0,r1,c0,c1);
      
      /* Compute determinant */
      determinant	= a*d - b*c;
   
      /* Check if transform is singular */
      if (determinant <= 0.000001)
	{
	  fprintf(stderr,"AffineTransform : Singular transform with det=%g\n",determinant);
	  exit(1);
	}

      /* Trap degenerate image */
      if ((rows == 1) && 
	  (cols == 1) && 
	  (a == d) && 
	  (determinant <= (1.0 + 0.000001)))
	degenerate = true;

      /* Compute inverse transform */
      ia	=   (d / determinant);
      ib	= - (b / determinant);
      ic	= - (c / determinant);
      id	=   (a / determinant);

      /* Get bounds of image */

      bounds[0][1]	= (double) c0;
      bounds[0][0]	= (double) r0;
      bounds[1][1]	= bounds[0][1] + (double)cols;
      bounds[1][0]	= bounds[0][0]; 
      bounds[2][1]	= bounds[0][1]; 
      bounds[2][0]	= bounds[0][0] + (double)rows;
      bounds[3][1]	= bounds[1][1];
      bounds[3][0]	= bounds[2][0];

      /* Compute new bounding box */
      x_min = x_max = a * (bounds[0][0]+o0) + b * (bounds[0][1]+o1);
      y_min = y_max = c * (bounds[0][0]+o0) + d * (bounds[0][1]+o1);

      {
	double	b_x, b_y;
	int	i;
      
	for(i = 1; i < 4; i++)
	  {
	    b_x  = a * (bounds[i][0]+o0) + b * (bounds[i][1]+o1);
	    b_y	 = c * (bounds[i][0]+o0) + d * (bounds[i][1]+o1);
	    x_min = MIN(x_min, b_x);
	    x_max = MAX(x_max, b_x);
	    y_min = MIN(y_min, b_y);
	    y_max = MAX(y_max, b_y);
	  }
      }
    
      x_start	= (int) (rint(x_min));
      x_end	= (int) (rint(x_max));
      y_start	= (int) (rint(y_min));
      y_end	= (int) (rint(y_max));
      
    
      if (degenerate)
	{
	  O = new array<U>(x_start,y_start,x_start+1,y_start+1);
	  _O = O->storage();
	  _O[x_start][y_start] = I[r0][c0];
	  return( O->temporary() );
	}

      O = new array<U>(x_start,x_end-1,y_start,y_end-1);

      _O = O->storage();
    
      /* Fill it */
      {
	double	back_x, back_y, t1, t2, t0;
	int     bx, by;
	double	v00, v01, v10, v11;
	int	xx, yy;
      
	for(x = x_min, xx = x_start; xx < x_end; x++, xx++)
	  {
	    for(y = y_min, yy = y_start; yy < y_end; y++, yy++)
	      {
		back_x = (ia * (x + 0.5) + ib * (y + 0.5)) - 0.5 - o0;
		back_y = (ic * (x + 0.5) + id * (y + 0.5)) - 0.5 - o1;
		bx     = (int)floor(back_x);
		by     = (int)floor(back_y);
	      
		v00 = v01 = v10 = v11 = 0;
	      
		/* Read surrounding values of back projection */
	      
		if ((r0 <= bx) && (bx <=r1))
		  {
		    if ((c0 <= by) && (by <= c1))
		      v00 = _I[bx][by];
		    if ((c0 <= by+1) && (by+1 <= c1))
		      v01 = _I[bx][by+1];
		  }
		if ((r0 <= bx+1) && (bx+1 <= r1))
		  {
		    if ((c0 <= by) && (by <= c1))
		      v10 = _I[bx+1][by];
		    if ((c0 <= by+1) && (by+1 <= c1))
		      v11 = _I[bx+1][by+1];
		  }

		t0  = (back_y - by);
		t1  = v00 + t0 * (v01 - v00);
		t2  = v10 + t0 * (v11 - v10);
	      
		_O[xx][yy] = (U)(t1+(back_x-bx)*(t2-t1));
	      }
	  }
      }
      return O->temporary();
    }
}

#endif
