#include	<arrayTools/Vector.h>
#include	<arrayTools/Array.h>
#include	<arrayTools/Average.h>
#include        <values.h>
#include        <stdio.h>
#include        <unistd.h>

namespace _ARRAYTOOLS_NAMESPACE
{

//! Compute the local image contrast by using a saturating 
//! mapping. The values lies in the interval [0,200]
//! A local arithmetic average  is used.
//!

array<u_char>&	Contrast(array<u_char>& I, double w)
 {
   array<double>	R, T;
   array<u_char>        *rI;
   int		        x,y;
   int                  x0, x1, y0, y1;
   double	        v;
   double               **_R, **_T;
   u_char               **_rI;
   
   x0   = I.first_col_index();
   x1   = I.last_col_index();
   y0   = I.first_row_index();
   y1   = I.last_row_index();

   rI   = new array<u_char>(y0,y1,x0,x1);
   
   R    = Cast(I,T);
   T	= GaussianSmooth(R, w);

   _R   = R.storage();
   _T   = T.storage();
   _rI  = (*rI).storage();
   
   for(y = y0; y <= y1; y++)
     for(x = x0; x <= x1; x++)
	{
	  if(_T[y][x] > 0.0)
	     {
	       v = _R[y][x] / _T[y][x];

	       if(v > 1.0)
			 v  = 2 - 1/v;
	     }
	  else
		{
		  v = 1.0;
		}
	  
	  _rI[y][x] = (unsigned char)(125*v);
	}
   
   return((*rI).temporary());
 }
  
  array<int>&	Contrast(array<int>& I, double w, double norm = 127.0)
  {
	array<int>	    R, T;
	array<int>     *rI;
	int		        x,y;
	int                  x0, x1, y0, y1;
	double	        v;
	double          norm2       = norm * norm;
	double          doubleNorm  = norm * 2.0;
	int           **_R, **_T;
	int           **_rI = NULL;
   
	x0   = I.first_col_index();
	x1   = I.last_col_index();
	y0   = I.first_row_index();
	y1   = I.last_row_index();

	rI   = new array<int>(y0,y1,x0,x1);
   
	R    = I;
	T	= GaussianSmooth(R, w);

	_R   = R.storage();
	_T   = T.storage();
	_rI  = (*rI).storage();
   
	for(y = y0; y <= y1; y++)
	  for(x = x0; x <= x1; x++)
		{
		  if(_T[y][x] > 0.0)
			{
			  v = (norm * _R[y][x]) / _T[y][x];

			  if(v > norm)
				v  = doubleNorm  - norm2/v;
			}
		  else
			{
			  v = norm;
			}
	  
		  _rI[y][x] = int(v+0.5);
		}
   
	return((*rI).temporary());
  }

array<float>&	Contrast(array<float>& I, double w, double norm = 127.0)
 {
   array<double>	R, T;
   array<float>        *rI;
   int		        x,y;
   int                  x0, x1, y0, y1;
   double	        v;
   double               **_R, **_T;
   float                **_rI = NULL;
   
   x0   = I.first_col_index();
   x1   = I.last_col_index();
   y0   = I.first_row_index();
   y1   = I.last_row_index();

   rI   = new array<float>(y0,y1,x0,x1);
   
   R    = Cast(I,T);
   T	= GaussianSmooth(R, w);

   _R   = R.storage();
   _T   = T.storage();
   _rI  = (*rI).storage();
   
   for(y = y0; y <= y1; y++)
     for(x = x0; x <= x1; x++)
	{
	  if(_T[y][x] > 0.0)
	     {
	       v = _R[y][x] / _T[y][x];

	       if(v > 1.0)
		 v  = 2 - 1/v;
	     }
	  else
	     {
	       v = 1.0;
	     }
	  
	  _rI[y][x] = (norm*v);
	}
   
   return((*rI).temporary());
 }

array<u_char>&	Contrast(array<u_char>& I, double w, double alpha)
 {
   array<double>	R, T;
   array<u_char>        *rI;
   int		        x,y;
   int                  x0, x1, y0, y1;
   double	        v;
   double               **_R, **_T;
   u_char               **_rI;
   
   x0   = I.first_col_index();
   x1   = I.last_col_index();
   y0   = I.first_row_index();
   y1   = I.last_row_index();

   rI   = new array<u_char>(y0,y1,x0,x1);
   
   R    = Cast(I,T);
   T	= GaussianSmooth(R, w);

   _R   = R.storage();
   _T   = T.storage();
   _rI  = (*rI).storage();
   
   for(y = y0; y <= y1; y++)
     for(x = x0; x <= x1; x++)
	{
	  if(_T[y][x] > 0.0)
	     {
	       v = _R[y][x] / _T[y][x];

	       if(v >= 1.0)
		 v  = 2 - 1/pow(v, alpha);
	       else
		 v  = pow(v, alpha);
	     }
	  else
	     {
	       v = 1.0;
	     }
	  
	  _rI[y][x] = (unsigned char)(125*v);
	}
   
   return((*rI).temporary());
 }



array<u_char>&	fastContrast(array<u_char>& I, int w)
 {
   array<u_char>        *rI, T;
   int		        x,y;
   int                  x0, x1, y0, y1;
   double	        v;
   u_char               **_rI, **_T;
   
   x0   = I.first_col_index();
   x1   = I.last_col_index();
   y0   = I.first_row_index();
   y1   = I.last_row_index();

   rI   = new array<u_char>(I);
   T	= Average((*rI), w, w);

   _T   = T.storage();
   _rI  = (*rI).storage();
   
   for(y = y0; y <= y1; y++)
     for(x = x0; x <= x1; x++)
	{
	  if(_T[y][x] > 0)
	     {
	       v = double(_rI[y][x]) / _T[y][x];

	       if(v > 1.0)
		 v  = 2 - 1/v;
	     }
	  else
	     {
	       v = 1.0;
	     }
	  
	  _rI[y][x] = (unsigned char)(125*v);
	}
   
   return((*rI).temporary());
 }


array<double>&	Contrast(array<double>& I, double w, double norm = 127.0)
 {
   array<double>	R, T;
   array<double>   *rI;
   int		        x,y;
   int              x0, x1, y0, y1;
   double	        v;
   double          norm2       = norm * norm;
   double          doubleNorm  = norm * 2.0;
   double         **_R, **_T;
   double         **_rI = NULL;
   
   x0   = I.first_col_index();
   x1   = I.last_col_index();
   y0   = I.first_row_index();
   y1   = I.last_row_index();

   rI   = new array<double>(y0,y1,x0,x1);
   
   R    = Cast(I,T);
   T	= GaussianSmooth(R, w);

   _R   = R.storage();
   _T   = T.storage();
   _rI  = (*rI).storage();
   
   for(y = y0; y <= y1; y++)
     for(x = x0; x <= x1; x++)
	   {
		 if(_T[y][x] > 0.0)
		   {
			 v = (norm * _R[y][x]) / _T[y][x];

			 if(v > norm)
			   v  = doubleNorm - norm2/v;
		   }
		 else
		   {
			 v = norm;
		   }
	  
		 _rI[y][x] = v;
	   }
   
   return((*rI).temporary());
 }

}
