
/*
  Support P4 P5 P6 images

*/


#include	<arrayTools/Array.h>
#include        <ctype.h>
#include        <stdio.h>
#include        <arrayTools/PpmIO.h>
#include        <arrayTools/Utils.h>
#include        <string.h>
#include        <unistd.h>


const int BUFFER_SIZE=2048;
const int PGM=1;
const int PPM=2;
const int PBM=3;
const unsigned char	mask[8] = {128, 64, 32, 16, 8, 4, 2, 1};


namespace _ARRAYTOOLS_NAMESPACE
{

  static int 
  rread(istream&  f,
	int       n,
	int       attempts,
	char     *buffer)
  {
    int togo = attempts;
    int got  = 0;

    while((got < n) && (togo > 0))
      {
	f.read(buffer+got, n - got);
	got  += f.gcount();
	togo -= 1;

	if(got != n)
	  usleep(10);
      }
     
    return(got);
  }

  static int 
  rget(istream&  f,
       int       attempts)
  {
    int  togo = attempts;
    char val;
     
    while(--togo > 0)
      {
	if(f.get(val))
	  return(int(val));
	else
	  usleep(40);
      }
     
    return(EOF);
  }

  /*
    int ReadPPMFileHeader(istream& f,
    int & width, 
    int & height, 
    int & maxIntensity)
    {
    char buffer[BUFFER_SIZE];
    int  i, numFields, firstInLine;
    int  c;
    int  type = 0;

    //! Read 4 space-separated fields from the file, ignoring
    //! comments (any line that starts with "#").
    //!
    c = rget(f, 100);
    firstInLine = 1;
    i           = 0;
    for (numFields = 0; numFields < 4; numFields++) 
    {
    //! Skip comments and white space.
    //!
	 
    while (1) {
    while (isspace(c)) {
    firstInLine = (c == '\n');
    c = rget(f, 100);
    }
    if (c != '#') {
    break;
    }
    do {
    c = rget(f, 100);
    } while ((c != EOF) && (c != '\n'));
    firstInLine = 1;
    }
	 
    //! Read a field (everything up to the next white space).
    //!
    while ((c != EOF) && !isspace(c)) 
    {
    if (i < (BUFFER_SIZE-2)) 
    {
    buffer[i]  = c;
    i++;
    }
    c = rget(f, 100);
    }
	 
    if (i < (BUFFER_SIZE-1)) 
    {
    buffer[i] = ' ';
    i++;
    }
    firstInLine = 0;
    }
    buffer[i] = 0;
    
    //! Parse the fields, which are: id, width, height, maxIntensity.
    //!
    if (strncmp(buffer, "P6 ", 3) == 0) 
    {
    type = PPM;
    } 
    else if (strncmp(buffer, "P5 ", 3) == 0) 
    {
    type = PGM;
    } 
    else 
    {
    return 0;
    }

    if (sscanf(buffer+3, "%d %d %d", &width, &height, &maxIntensity)
    != 3) 
    {
    return 0;
    }
    return type;
    }

  */





  int scan_for_fields(const char * input, int len, char * buffer)
  {
    int i = 0;
    int j = 0;
    const char * ptr = input;

    int c = *ptr++;
    ++j;

    while (j < len) {
      while (isspace(c) && j < len) {
	c = *ptr++;
	++j;
      }
      if (c != '#') {
	break;
      }
      do {
	c = *ptr++;
	++j;
      } while ((c != EOF) && (c != '\n') && (j < len));
    }
	 
    //! Read a field (everything up to the next white space).
    //!
    while ((c != EOF) && !isspace(c) && j < len) 
      {
	if (i < (BUFFER_SIZE-1)) 
	  {
	    buffer[i]  = c;
	    i++;
	  }
	c = *ptr++;
	++j;
      }
	 
    if (i < BUFFER_SIZE) 
      {
	buffer[i] = 0;
      }
    return j;
  }

  void scan_for_fields(istream & f, char * buffer)
  {
    int i = 0;
    int c = rget(f,100);

    while (1) {
      while (isspace(c)) {
	c = rget(f, 100);
      }
      if (c != '#') {
	break;
      }
      do {
	c = rget(f, 100);
      } while ((c != EOF) && (c != '\n'));
    }
	 
    //! Read a field (everything up to the next white space).
    //!
    while ((c != EOF) && !isspace(c)) 
      {
	if (i < (BUFFER_SIZE-1)) 
	  {
	    buffer[i]  = c;
	    i++;
	  }
	c = rget(f, 100);
      }
	 
    if (i < BUFFER_SIZE) 
      {
	buffer[i] = 0;
      }
  }

  int ReadPPMFileHeader(istream& f,
			int & width, 
			int & height, 
			int & maxIntensity)
  {
    char buffer[BUFFER_SIZE];
    int  type = 0;
    
    scan_for_fields(f,buffer);
        
    //! Parse the fields, which are: id, width, height, maxIntensity.
    //!
    if (strncmp(buffer, "P6", 2) == 0) 
      {
	type = PPM;
      } 
    else if (strncmp(buffer, "P5", 2) == 0) 
      {
	type = PGM;
      } 
    else if (strncmp(buffer,"P4",2) == 0)
      {
	type = PBM;
      }
    else
      {
	return 0;
      }


    scan_for_fields(f,buffer);
    if( sscanf(buffer,"%d",&width) != 1) return 0;
    scan_for_fields(f,buffer);
    if( sscanf(buffer,"%d",&height) != 1) return 0;

    if(type == PBM)
      {
	maxIntensity = 0;
      }
    else
      {
	scan_for_fields(f,buffer);
	if( sscanf(buffer,"%d",&maxIntensity) != 1) return 0;
      }

    return type;
  }

  int ReadPPMFileHeader(const char * input,
			int len,
			int & width, 
			int & height, 
			int & maxIntensity,
			int & type)
  {
    type = 0;
    char buffer[BUFFER_SIZE];
    int n = 0;
    int l = len;

    const char * ptr = input;

    n = scan_for_fields(ptr, l, buffer);
    ptr += n;
    l -= n;
    
    //! Parse the fields, which are: id, width, height, maxIntensity.
    //!
    if (strncmp(buffer, "P6", 2) == 0) 
      {
	type = PPM;
      } 
    else if (strncmp(buffer, "P5", 2) == 0) 
      {
	type = PGM;
      } 
    else if (strncmp(buffer,"P4",2) == 0)
      {
	type = PBM;
      }
    else
      {
	return 0;
      }


    n = scan_for_fields(ptr, l, buffer);
    ptr += n;
    l -= n;

    if( sscanf(buffer,"%d",&width) != 1) return 0;

    n = scan_for_fields(ptr,l,buffer);
    ptr += n;
    l -= n;

    if( sscanf(buffer,"%d",&height) != 1) return 0;

    if(type == PBM)
      {
	maxIntensity = 0;
      }
    else
      {
	n = scan_for_fields(ptr, l,buffer);
	ptr += n;
	l -= n;
	if( sscanf(buffer,"%d",&maxIntensity) != 1) return 0;
      }

    return len - l;
  }






  static inline int CheckArrayDims(array<unsigned char> &A, 
				   int            y0, 
				   int            y1,
				   int            x0, 
				   int            x1)
  {
    if((A.first_row_index() == y0) && (A.last_row_index() == y1) &&
       (A.first_col_index() == x0) && (A.last_col_index() == x1))
      {
	return(1);
      }
    else
      {
	return(0);
      }
  }

  //! Read a PPM from stream: 
  //!

  int ReadPpm(array<unsigned char>& R, 
	      array<unsigned char>& G,
	      array<unsigned char>& B,
	      istream&       s)
  {
    int type;
    int width;
    int height;
    int max_value;
   
    //! Read PPM Header from stream 
    //!
    type = ReadPPMFileHeader(s, width, height, max_value);
   
    if(type != PPM)
      {
	return FALSE;
      }
    else
      {
	unsigned char *buf = NULL, *prgb;
	unsigned char **_R, **_G, **_B;
	
	//! Create temporary buffer
	//!
	buf  = new unsigned char[height*width*3];
	prgb = buf;
	
	//! Get all data into buffer
	//!
	rread(s, height*width*3, 100, (char *) buf);
	
	//! Allocate the return arrays
	//!
	if(!CheckArrayDims(R, 0, height-1, 0, width-1))
	  R = (* new array<unsigned char>(height, width)).temporary();

	if(!CheckArrayDims(G, 0, height-1, 0, width-1))
	  G = (* new array<unsigned char>(height, width)).temporary();
	
	if(!CheckArrayDims(B, 0, height-1, 0, width-1))
	  B = (* new array<unsigned char>(height, width)).temporary();

	_R = R.storage();
	_G = G.storage();
	_B = B.storage();
	
	//! Read RGB triplets from file into the corresponding arrays
	//!
	if(max_value != 255)
	  {
	    int i, j;
	   
	    for(j = 0; j < height; j++)
	      {
		for(i = 0; i < width; i++)
		  {
		    _R[j][i] = (255*(*prgb++))/max_value;
		    _G[j][i] = (255*(*prgb++))/max_value;
		    _B[j][i] = (255*(*prgb++))/max_value;
		  }
	      }
	  }
	else
	  {
	    int i, j;
	   
	    for(j = 0; j < height; j++)
	      {
		for(i = 0; i < width; i++)
		  {
		    _R[j][i] = (*prgb++);
		    _G[j][i] = (*prgb++);
		    _B[j][i] = (*prgb++);
		  }
	      }
	  }
	
	delete buf;
      }
   
    return TRUE;
  }
  
  //! Read a PGM from stream: 
  //!

  int ReadPgm(array<unsigned char>& I, 
	      istream&       s)
  {
    int type;
    int width;
    int height;
    int max_value;
   
    //! Read PPM Header from stream 
    //!
    type = ReadPPMFileHeader(s, width, height, max_value);
   
    if(type != PGM)
      {
	return FALSE;
      }
    else
      {
	unsigned char *buf   = NULL;
	unsigned char *pi;
	unsigned char **_I   = NULL;
	int     size  = height*width;

	//! Create temporary buffer
	//!
	buf  = new unsigned char[size];
	pi   = buf;
	
	//! Get all data into buffer
	//!
	s.read((char *)buf, size);
	

	//! Allocate the return arrays
	//!
	I  = (* new array<unsigned char>(height, width)).temporary();
	_I = I.storage();
	
	//! Read luminance values from file into the corresponding array
	//!
	if(max_value != 255)
	  {
	    int i, j;
	   
	    for(j = 0; j < height; j++)
	      {
		for(i = 0; i < width; i++)
		  {
		    _I[j][i] = (255*(*pi++))/max_value;
		  }
	      }
	  }
	else
	  {
	    int i, j;
	   
	    for(j = 0; j < height; j++)
	      {
		for(i = 0; i < width; i++)
		  {
		    _I[j][i] = (*pi++);
		  }
	      }
	  }
	

	delete buf;
      }
   
    return TRUE;
  }

  int ReadPbm(array<unsigned char>& I, 
	      istream&       s, 
	      int foreground_value, int background_value)
  {
    int type;
    int width;
    int height;
    int max_value;
   
    //! Read PPM Header from stream 
    //!
    type = ReadPPMFileHeader(s, width, height, max_value);
   
    if(type != PBM)
      {
	return FALSE;
      }
    else
      {
	int r,c,k,h;
	const unsigned char	mask[8] = {128, 64, 32, 16, 8, 4, 2, 1};
	unsigned char *buf   = NULL;
       	unsigned char **_I   = NULL;
	int     size  = width/8;

	if (width % 8) size++;

	//! Create temporary buffer
	//!
	buf  = new unsigned char[size];

	//! Allocate the return arrays
	//!
	I  = (* new array<unsigned char>(height, width)).temporary();
	_I = I.storage();
	
	//! Read luminance values from file into the corresponding array
	//!

	for(r = 0; r < height; r++)
	  {
	    s.read(reinterpret_cast<char*>(buf),size);
	    c = 0;
	    for(k = 0; k < size; k++)
	      {
		for(h = 0; h < 8; h++)
		  {
		    if(c >= width) break;
		    _I[r][c] = ( buf[k] & mask[h] ) ? foreground_value : background_value;
		    c++;
		  }
	      }
	  }
	  
	delete buf;
      }
   
    return TRUE;
  }

  int ReadPpm(array<unsigned char>& R, 
	      array<unsigned char>& G,
	      array<unsigned char>& B,
	      const char * input, 
	      int len)
  {
    int type;
    int width;
    int height;
    int max_value;

    if(input == NULL) return FALSE;
   

    const char * ptr = input;
    int n;

    n = ReadPPMFileHeader(input, len, width, height, max_value, type);
    ptr += n;

    if(type != PPM)
      {
	return FALSE;
      }
    else
      {
      	unsigned char **_R, **_G, **_B;
	
	//! Allocate the return arrays
	//!
	if(!CheckArrayDims(R, 0, height-1, 0, width-1))
	  R = (* new array<unsigned char>(height, width)).temporary();

	if(!CheckArrayDims(G, 0, height-1, 0, width-1))
	  G = (* new array<unsigned char>(height, width)).temporary();
	
	if(!CheckArrayDims(B, 0, height-1, 0, width-1))
	  B = (* new array<unsigned char>(height, width)).temporary();

	_R = R.storage();
	_G = G.storage();
	_B = B.storage();
	
	//! Read RGB triplets from file into the corresponding arrays
	//!
	if(max_value != 255)
	  {
	    int i, j;
	   
	    for(j = 0; j < height; j++)
	      {
		for(i = 0; i < width; i++)
		  {
		    _R[j][i] = (255*(*ptr++))/max_value;
		    _G[j][i] = (255*(*ptr++))/max_value;
		    _B[j][i] = (255*(*ptr++))/max_value;
		  }
	      }
	  }
	else
	  {
	    int i, j;
	   
	    for(j = 0; j < height; j++)
	      {
		for(i = 0; i < width; i++)
		  {
		    _R[j][i] = (*ptr++);
		    _G[j][i] = (*ptr++);
		    _B[j][i] = (*ptr++);
		  }
	      }
	  }
      }
   
    return TRUE;
  }
  
  //! Read a PGM from stream: 
  //!

  int ReadPgm(array<unsigned char>& I, 
	      const char * input,
	      int len)
  {
    int type;
    int width;
    int height;
    int max_value;
    int n = 0;
    const char * ptr = input;

    //! Read PPM Header from stream 
    //!
    n = ReadPPMFileHeader(ptr, len, width, height, max_value, type);
    ptr += n;

    if(type != PGM)
      {
	return FALSE;
      }
    else
      {
	unsigned char **_I   = NULL;

	//! Allocate the return arrays
	//!
	I  = (* new array<unsigned char>(height, width)).temporary();
	_I = I.storage();
	
	//! Read luminance values from file into the corresponding array
	//!
	if(max_value != 255)
	  {
	    int i, j;
	   
	    for(j = 0; j < height; j++)
	      {
		for(i = 0; i < width; i++)
		  {
		    _I[j][i] = (255*(*ptr++))/max_value;
		  }
	      }
	  }
	else
	  {
	    int i, j;
	   
	    for(j = 0; j < height; j++)
	      {
		for(i = 0; i < width; i++)
		  {
		    _I[j][i] = (*ptr++);
		  }
	      }
	  }
      }
   
    return TRUE;
  }

 

  //! Write PPM to stream
  //!

  int WritePpm(array<unsigned char>& R, 
	       array<unsigned char>& G,
	       array<unsigned char>& B,
	       ostream&      s)
  {

     if(!AreConsistent(R, G) ||  
	!AreConsistent(R, B) ||
	R.is_empty())
      {
	return FALSE;
      }
    

    char                 *ppm_magic = "P6";
    unsigned int width   = R.width();
    unsigned int height  = R.height();
   
    
    //! Write PPM magic, image width and height to stream
    //!
    {
      char buf[1024];
      
      sprintf(buf, "%s\n%d %d\n%d\n",
	      ppm_magic, width, height, 255);
      
      s.write(buf, strlen(buf));
    }
   
    s.flush();
   
    //! Write RGB triplets to file 
    //!
    {
      int i, j;
      int x0, x1, y0, y1;
      
      x0 = R.first_col_index();
      x1 = R.last_col_index();

      y0 = R.first_row_index();
      y1 = R.last_row_index();
      
      for(j = y0; j <= y1; j++)
	{
	  for(i = x0; i <= x1; i++)
	    {
	      unsigned char rgb[3];
		
	      rgb[0] = R[j][i];
	      rgb[1] = G[j][i];
	      rgb[2] = B[j][i];
		
	      s.write((char *) rgb, 3);
		
	      /*
		s.write((char *)&(rgb[0]), 1);
		s.write((char *)&(rgb[1]), 1);
		s.write((char *)&(rgb[2]), 1);
	      */
	    }
	}
    }
   
    DeleteIfTemporary(R);
    DeleteIfTemporary(G);
    DeleteIfTemporary(B);
   
    s.flush();
   
    return TRUE;
  }

  int WritePgm(array<unsigned char>& I, 
	       ostream&      s)
  {

    if(I.is_empty())
      {
	return FALSE;
      }

    char                 *pgm_magic = "P5";
    unsigned int width   = I.width();
    unsigned int height  = I.height();
   
    //! Write PGM magic, image width and height to stream
    //!
    {
      char *buf;

      buf = new char [1024];
      
      sprintf(buf, "%s\n%d %d\n%d\n",
	      pgm_magic, width, height, 255);
      
      s.write(buf, strlen(buf));

      delete [] buf;
    }
   
    s.flush();
   
    //! Write I values to file
    //!
    {
      int i, j;
      int x0, x1, y0, y1;
      
      x0 = I.first_col_index();
      x1 = I.last_col_index();

      y0 = I.first_row_index();
      y1 = I.last_row_index();
      
      for(j = y0; j <= y1; j++)
	{
	  for(i = x0; i <= x1; i++)
	    {
	      unsigned char l;
		
	      l = I[j][i];

	      s.write((char *)&l, 1);
	    }
	}
    }
   
    DeleteIfTemporary(I);
   
    s.flush();
   
    return TRUE;
  }

  int WritePbm(array<unsigned char>& I, 
	       ostream&      s)
  {
    if(I.is_empty())
      {
	return FALSE;
      }
    
    char                 *pgm_magic = "P4";
    unsigned int width   = I.width();
    unsigned int height  = I.height();
   
    {
      char *buf;
     
      buf = new char [1024];
     
      sprintf(buf, "%s\n%d %d\n",
	      pgm_magic, width, height);
     
      s.write(buf, strlen(buf));
     
      delete [] buf;
    }
   
    s.flush();
   
    //! Write I values to file
    //!
    {
      int i, j, k, m;
      int r0, r1, c0, c1;

      I.boundary(r0,r1,c0,c1);
      
      int bufsize = width / 8;
      if (width % 8) bufsize++;

      unsigned char * temp = new unsigned char[bufsize];
      unsigned char ** _I = I.storage();

      for (i = r0; i <= r1; i++) 
	{
	  j = c0;
	  for (k = 0; k < bufsize; k++) 
	    {
	      temp[k] = 0;
	      for (m = 0; m < 8; m++) 
		{
		  if(j > c1) break;
		  temp[k] |= ( (_I[i][j]) ? 0 : mask[m]);
		  j++;
		}
	    }
	  s.write(reinterpret_cast<char*>(temp),bufsize);	  
        }
    
      delete temp;
    }      
    DeleteIfTemporary(I);
    
    s.flush();
    
    return TRUE;
  }
  




  /*
   * Same functions using namefile and handling file open-close
   */

  int ReadPpm(array<unsigned char>& R, 
	      array<unsigned char>& G,
	      array<unsigned char>& B,
	      const char * filename)
  {
    if(filename == NULL) return FALSE;
    ifstream ifs(filename);

    if(ifs.is_open())
      {
	int ok = ReadPpm(R,G,B,ifs);
	ifs.close();
	return ok;
      }
    else
      {
	return FALSE;
      }
  }


  int ReadPgm(array<unsigned char>& I, 
	      const char * filename)
  {
    if(filename == NULL) return FALSE;
    ifstream ifs(filename);
    if(ifs.is_open())
      {
	int ok = ReadPgm(I,ifs);
	ifs.close();
	return ok;
      }
    else
      {
	return FALSE;
      }
  }

  int ReadPbm(array<unsigned char>& I, 
	      const char * filename,
	      int foreground_value, int background_value)
  {
    if(filename == NULL) return FALSE;
    ifstream ifs(filename);
    if(ifs.is_open())
      {
	int ok = ReadPbm(I,ifs,foreground_value, background_value);
	ifs.close();
	return ok;
      }
    else
      {
	return FALSE;
      }
  }

  int WritePpm(array<unsigned char>& R, 
	       array<unsigned char>& G,
	       array<unsigned char>& B,
	       const char * filename)
  {
    if(filename == NULL) return FALSE;
    ofstream ofs(filename);
    if(ofs.is_open())
      {
	int ok = WritePpm(R,G,B,ofs);
	ofs.close();
	return ok;
      }
    else
      {
	return FALSE;
      }
  }

  int WritePgm(array<unsigned char>& I, 
	       const char * filename)
  {
    if(filename == NULL) return FALSE;
    ofstream ofs(filename);
    if(ofs.is_open())
      {
	int ok = WritePgm(I,ofs);
	ofs.close();
	return ok;
      }
    else
      {
	return FALSE;
      }
  }

  int WritePbm(array<unsigned char>& I, 
	       const char * filename)
  {
    if(filename == NULL) return FALSE;
    ofstream ofs(filename);
    if(ofs.is_open())
      {
	int ok = WritePbm(I,ofs);
	ofs.close();
	return ok;
      }
    else
      {
	return FALSE;
      }
  }
}
