#include <arrayTools/Array.h>
#include <vector>
#include <unistd.h>
#include <tiffio.h>

namespace _ARRAYTOOLS_NAMESPACE
{
  int TiffTest()
  {
	return(1);
  }

  int WriteTiffRGBA(array<u_char> &R,
					array<u_char> &G,
					array<u_char> &B,
					array<u_char> &A,
					char        *name)
  {
	TIFF *out = TIFFOpen(name, "w");
	int   width = R.cols();
	int   height = R.rows();
	int   sampleperpixel = 4;
	int   x0, x1, y0, y1;
	tsize_t linebytes = sampleperpixel * width;    
	unsigned char *buf = NULL; 

	R.stat(x0, x1, y0, y1);

	char *image = new char [width*height*sampleperpixel], *p;

	p = image;
	for(int y = y0; y <= y1; y++)
	  {
		for(int x = x0; x <= x1; x++)
		  {
			*p++ = R[y][x];
			*p++ = G[y][x];
			*p++ = B[y][x];
			*p++ = A[y][x];
		  }
	  }
      
	TIFFSetField(out, TIFFTAG_IMAGEWIDTH, width);      // set the width of the image
	TIFFSetField(out, TIFFTAG_IMAGELENGTH, height);    // set the height of the image
	TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, sampleperpixel); // set number of channels per pixel
	TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, 8);       // set the size of the channels
	TIFFSetField(out, TIFFTAG_EXTRASAMPLES, EXTRASAMPLE_ASSOCALPHA);// set the size of the channels
	TIFFSetField(out, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);    // set the origin of the image.
	TIFFSetField(out, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
	TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);

	buf = (unsigned char *)_TIFFmalloc(TIFFScanlineSize(out));
  
	// We set the strip size of the file to be size of one row of pixels
	//
	TIFFSetField(out, TIFFTAG_ROWSPERSTRIP, TIFFDefaultStripSize(out, width*sampleperpixel));
  
	// Now writing image to the file one strip at a time
	//
	{
	  uint32 uheight = (uint32)height;

	  for (uint32 row = 0; row < uheight; row++)
		{
		  memcpy(buf, &image[(height-row-1)*linebytes], linebytes); 
		  if (TIFFWriteScanline(out, buf, row, 0) < 0)
			break;
		}
	}
	(void) TIFFClose(out);

	if (buf)
	  _TIFFfree(buf);


	// release temporary image buffer
	//
	delete image;

	return(0);
  }

  int ReadTiffRGBA(array<u_char> &R,
				   array<u_char> &G,
				   array<u_char> &B, 
				   array<u_char> &Alfa, 
				   char        *fileName)
  {
	TIFF* tif = TIFFOpen(fileName, "r"); 
  
	if(tif) 
	  { 
		TIFFRGBAImage img; 
		char          emsg[1024]; 

		if (TIFFRGBAImageBegin(&img, tif, 0, emsg))
		  { 
			size_t   npixels; 
			uint32*  raster; 

			npixels = img.width * img.height;
			raster = (uint32*) _TIFFmalloc(npixels * sizeof (uint32)); 

			if (raster != NULL) 
			  { 
				if (TIFFRGBAImageGet(&img, raster, img.width, img.height))
				  { 
					int     w = img.width;
					int     h = img.height;
					uint32 *P = raster;

					R  = (*(new array<u_char> (img.height, img.width))).temporary();	      
					G  = (*(new array<u_char> (img.height, img.width))).temporary();	      
					B  = (*(new array<u_char> (img.height, img.width))).temporary();	      
					Alfa  = (*(new array<u_char> (img.height, img.width))).temporary();	      
		  
					for(int y = h-1; y >= 0; y--)
					  for(int x = 0; x < w; x++)
						{
						  u_char *p = (u_char *) P;
			
						  R[y][x] = p[0];
						  G[y][x] = p[1];
						  B[y][x] = p[2];
						  Alfa[y][x] = p[3];
						  P++;
						}
				  }

				_TIFFfree(raster);
			  }
	  
			TIFFRGBAImageEnd(&img); 
		  }
		else
		  {
			TIFFError(fileName, emsg);
		  }

		TIFFClose(tif); 
	  }

	return(0);
  }

  int ReadTiffFloatRGB(array<float> &I1,
					   array<float> &I2,
					   array<float> &I3,
					   char       *name) 
  { 
	TIFF* tif = TIFFOpen(name, "r"); 

	if (tif) 
	  { 
		uint32 imagelength; 
		uint32 imagewidth; 
		tdata_t buf; 
		uint32 row; 
		float  *p;

		TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &imagelength); 
		TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &imagewidth); 

		I1 = (* new array<float> (imagelength,  imagewidth)).temporary();
		I2 = (* new array<float> (imagelength,  imagewidth)).temporary();
		I3 = (* new array<float> (imagelength,  imagewidth)).temporary();

		buf = _TIFFmalloc(TIFFScanlineSize(tif)); 
		p   = (float *) buf;

		for (row = 0; row < imagelength; row++) 
		  {
			TIFFReadScanline(tif, buf, row);
	  
			for(int x = 0; x < int(imagewidth); x++)
			  {
				I1[row][x] = (*p++);
				I2[row][x] = (*p++);
				I3[row][x] = (*p++);
			  }

			p   = (float *) buf;
		  }

		_TIFFfree(buf); 
		TIFFClose(tif); 
	  } 

	return(0);
  }

  int ReadTiffFloatRGBA(array<float> &I1,
						array<float> &I2,
						array<float> &I3,
						array<float> &I4,
						char       *name) 
  { 
	TIFF* tif = TIFFOpen(name, "r"); 

	if (tif) 
	  { 
		uint16 samplesperpixel;
		uint32 imagelength; 
		uint32 imagewidth; 
		tdata_t buf; 
		uint32 row; 
		float  *p;

		TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &samplesperpixel); 
		TIFFGetField(tif, TIFFTAG_IMAGELENGTH,     &imagelength); 
		TIFFGetField(tif, TIFFTAG_IMAGEWIDTH,      &imagewidth); 

		I1 = (* new array<float> (imagelength,  imagewidth)).temporary();
		I2 = (* new array<float> (imagelength,  imagewidth)).temporary();
		I3 = (* new array<float> (imagelength,  imagewidth)).temporary();
		I4 = (* new array<float> (imagelength,  imagewidth)).temporary();

		buf = _TIFFmalloc(TIFFScanlineSize(tif)); 
		p   = (float *) buf;

		for (row = 0; row < imagelength; row++) 
		  {
			TIFFReadScanline(tif, buf, row);
	  
			for(int x = 0; x < int(imagewidth); x++)
			  {
				I1[row][x] = (*p++);
				I2[row][x] = (*p++);
				I3[row][x] = (*p++);

				if(samplesperpixel == 4)
				  I4[row][x] = (*p++);
				else
				  I4[row][x] = 1.00;
			  }

			p   = (float *) buf;
		  }

		_TIFFfree(buf); 
		TIFFClose(tif); 
	  } 

	return(0);
  } 
}
