#include <arrayTools/aniCommon.h>
#include <arrayTools/Correlation.h>

extern "C"  {

  char corrType[4] = {'S', 'G', 'L', 'P'};

  // Wrapper for the arrayTools correlation function
  //
  SEXP aniCorrelation(SEXP animage_I, // image where the template is searched for (REAL)
					  SEXP animage_T, // templated to be located                  (REAL)
					  SEXP i_dd,      // size of neighborhood                     (INTEGER)
					  SEXP i_kind,    // correlation variant                      (INTEGER)
					  SEXP i_normalize,
					  SEXP i_p,
					  SEXP r_maxD,
					  SEXP animage_F, // if not FALSE, computation filter         (INTEGER)
					  SEXP animage_M) // if not FALSE, template filter weights: normalized to sum 1 (REAL)
  {
	SEXP   ani_r;
	int    dd        = INTEGER(i_dd)[0];
	int    kind      = INTEGER(i_kind)[0];
	int    normalize = INTEGER(i_normalize)[0];
	int    p         = INTEGER(i_p)[0];
	double maxD      = REAL(r_maxD)[0];

	ia::array<double>  I  = r2ia_rd(animage_I);
	ia::array<double>  T  = r2ia_rd(animage_T);
	ia::array<double> *pF = NULL;
	ia::array<double> *pM = NULL;
	ia::array<double>  R;

	if(TYPEOF(animage_F) == S4SXP) {
	  pF = &(r2ia_rd(animage_F));
	}

	if(TYPEOF(animage_M) == S4SXP) {
	  pM = &(r2ia_rd(animage_M));
	}

	int    rx;
	int    ry;
	double mr;

	R = CorrelationW2(I,
					  T,
					  dd,
					  corrType[kind],
					  normalize,
					  p,
					  maxD,
					  pF,
					  pM,
					  &rx,
					  &ry,
					  &mr);

	PROTECT(ani_r = NEW_LIST(4));

	SET_ELEMENT(ani_r, 0, ia2r_dr(R));
	SET_ELEMENT(ani_r, 1, ScalarInteger(rx));
	SET_ELEMENT(ani_r, 2, ScalarInteger(ry));
	SET_ELEMENT(ani_r, 3, ScalarReal(mr));

	UNPROTECT(1);

	return(ani_r);
  }
}
