#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/*
#include <R.h>
#include <Rdefines.h>
#include <Rinternals.h>
*/

#include <arrayTools/aniCommon.h>
#include <arrayTools/aniOp.h>
#include <arrayTools/aniOpImmediate.h>

// Utility function to get filter pixel value
//
static int get_pixel(animage &img,
					 int      y,
					 int      x)
{
  if((x < img.x) || ((img.x + img.w) <= x) ||
	 (y < img.y) || ((img.x + img.h) <= y))
	{
	  return(img.io);
	}
  else 
	{
	  return(img.idata[(x-img.x)*img.h+(y-img.y)]);
	}
	 
  return(0);
}

extern "C"  {

  SEXP aniOpInt_im_im_mk(SEXP animage_1,
						 SEXP ani_op,
						 SEXP animage_2,
						 SEXP animask)
  {
	SEXP ani_r;
    int	 XX[6], YY[6], REGIONS[5][5];
	int  x1, y1, w1, h1;
	int  x2, y2, w2, h2;
    int  xm, ym, wm, hm;
	int  o1, o2;

    // localize images 
	//
	x1 = GET_ANIMAGE_X(animage_1);
	y1 = GET_ANIMAGE_Y(animage_1);
	w1 = GET_ANIMAGE_W(animage_1);
	h1 = GET_ANIMAGE_H(animage_1);
	o1 = GET_ANIMAGE_OI(animage_1);

	x2 = GET_ANIMAGE_X(animage_2);
	y2 = GET_ANIMAGE_Y(animage_2);
	w2 = GET_ANIMAGE_W(animage_2);
	h2 = GET_ANIMAGE_H(animage_2);
	o2 = GET_ANIMAGE_OI(animage_2);

	// and mask
	//
	xm = GET_ANIMASK_X(animask);
	ym = GET_ANIMASK_Y(animask);
    wm = GET_ANIMASK_W(animask);
    hm = GET_ANIMASK_H(animask);

    // clearing REGIONS array 
	//
    for(int i = 0; i < 5; i++)
	  for(int j = 0; j < 5; j++)
	    REGIONS[i][j] = 0;
	    
    // compute table of delimiting coordinates of REGIONS 
	//
	XX[0] = xm;
	XX[1] = x1;
	XX[2] = x2;
	XX[3] = xm+wm;
	XX[4] = x1+w1;
	XX[5] = x2+w2;
     
	sorting(XX);

	YY[0] = ym;
	YY[1] = y1;
	YY[2] = y2;
	YY[3] = ym+hm;
	YY[4] = y1+h1;
	YY[5] = y2+h2;
	
	sorting(YY);

	/* 
	   REGIONS[i][j]  = 0 : out_of_matrix;
	   = 1 : image1 out of mask;
	   = 2 : image2 out of mask;
	   = 3 : image1 op image2 out of mask;
	   = 4 : mask;
	   = 5 : image1 in mask;
	   = 6 : image2 in mask;
	   = 7 : image1 op image2 in mask;
	*/
    
    for(int i = 0; i < 5; i++)
	  for(int j = 0; j < 5; j++)
	    if((XX[j] >= x1) && (XX[j] < x1+w1) &&
	       (YY[i] >= y1) && (YY[i] < y1+h1))
		  REGIONS[i][j] += 1;
	
    for(int i = 0; i < 5; i++)
	  for(int j = 0; j < 5; j++)
	    if((XX[j] >= x2) && (XX[j] < x2+w2) && 
	       (YY[i] >= y2) && (YY[i] < y2+h2))
		  REGIONS[i][j] += 2;

    for(int i = 0; i < 5; i++)
	  for(int j = 0; j < 5; j++)
	    if((XX[j] >= xm) && (XX[j] < xm+wm) && 
	       (YY[i] >= ym) && (YY[i] < ym+hm))
		  REGIONS[i][j] += 4;

	// compute outside of result
	//
	int opCode = statOpCode(CHAR(STRING_ELT(ani_op, 0)));
	int oo     = 0;

	switch (opCode)
	  {
	  case PLUS:          oo = o1 + o2; break;
	  case MINUS:         oo = o1 - o2; break;
	  case TIMES:         oo = o1 * o2; break;
	  case DIVIDE:        oo = (o2 > 0) ? o1 / o2 : 0; break;
	  case MAX:           oo = max(o1, o2); break;
	  case MIN:           oo = min(o1, o2); break;
	  case LESS_OR_EQ:    oo = (o1 <= o2) ? 1 : 0; break; 
	  case LESS:          oo = (o1 <  o2) ? 1 : 0; break; 
	  case GREATER_OR_EQ: oo = (o1 >= o2) ? 1 : 0; break; 
	  case GREATER:       oo = (o1 >  o2) ? 1 : 0; break; 
	  case EQUAL:         oo = (o1 == o2) ? 1 : 0; break; 
	  case AND:           oo = o1 & o2; break; 
	  case OR:            oo = o1 | o2; break;
	  case XOR:           oo = o1 ^ o2; break;
	  default:
		UNPROTECT(1);
		return(R_NilValue);
	  }

	// Create return animage
	//
	PROTECT(ani_r = newAnimage(xm, ym, wm, hm, int(oo)));

	// and get the needed pointers to storage
	//
	int *wpointer_s  = GET_ANIMAGE_DI(ani_r);
	int *rpointer1_s = GET_ANIMAGE_DI(animage_1);
	int *rpointer2_s = GET_ANIMAGE_DI(animage_2) ;

	for(int i = 0; i < 5; i++) 
	  {
		int deltaYY = YY[i+1] - YY[i];
		
		for(int j = 0; j < 5; j++) 
		  {
			int deltaXX = XX[j+1] - XX[j];

			int *wpointer  = wpointer_s  + (XX[j] - xm)*hm + (YY[i]-ym);
			int *rpointer1 = rpointer1_s + (XX[j] - x1)*h1 + (YY[i]-y1);
			int *rpointer2 = rpointer2_s + (XX[j] - x2)*h2 + (YY[i]-y2);

			if((deltaXX > 0) && (deltaYY > 0))
			  {
				switch(REGIONS[i][j]) {
				case 0:
				case 1:
				case 2:
				case 3:
				  break;
				case 4:
				  for(int h = 0; h < deltaXX; h++) {
					for(int k = 0; k < deltaYY; k++) {     
					  *wpointer++ = oo;
					}                                   
					wpointer  += hm - deltaYY;          
				  }
				  break;
				case 5: 
				  switch(opCode) {
				  case PLUS:          ARITH1_OPER_MASK(NOF, +);     break;
				  case MINUS:         ARITH1_OPER_MASK(NOF, -);     break;
				  case TIMES:         ARITH1_OPER_MASK(NOF, *);     break;
				  case DIVIDE:        
					if(o2)
					  {
						for(int h = 0; h < deltaXX; h++) {
						  for(int k = 0; k < deltaYY; k++) {     
							*wpointer++ = *rpointer1 / o2;
							rpointer1++;
						  }                                   
						  wpointer  += hm - deltaYY;   
						  rpointer1 += h1 - deltaYY;
						}
					  }
					else 
					  {
						for(int h = 0; h < deltaXX; h++) {
						  for(int k = 0; k < deltaYY; k++) {     
							*wpointer++ = 0;
						  }                                   
						  wpointer  += hm - deltaYY;          
						}
					  }
					break;
				  case MAX:           ARITH1_OPER_MASK(max, COMMA); break;
				  case MIN:           ARITH1_OPER_MASK(min, COMMA); break;
				  case LESS_OR_EQ:    ARITH1_OPER_MASK(NOF, <=);    break;
				  case LESS:          ARITH1_OPER_MASK(NOF, <);     break;
				  case GREATER_OR_EQ: ARITH1_OPER_MASK(NOF, >=);    break;
				  case GREATER:       ARITH1_OPER_MASK(NOF, >);     break;
				  case EQUAL:         ARITH1_OPER_MASK(NOF, ==);    break;
				  case AND:           ARITH1_OPER_MASK(NOF, &);     break;
				  case OR:            ARITH1_OPER_MASK(NOF, |);     break;
				  case XOR:           ARITH1_OPER_MASK(NOF, ^);     break; 
				  default:
					UNPROTECT(1);
					return(R_NilValue);
				  }
				  break;
				case 6:
				  switch(opCode) {
				  case PLUS:          ARITH2_OPER_MASK(NOF, +);     break;
				  case MINUS:         ARITH2_OPER_MASK(NOF, -);     break;
				  case TIMES:         ARITH2_OPER_MASK(NOF, *);     break;
				  case DIVIDE:        
					if(o1)
					  {
						for(int h = 0; h < deltaXX; h++) {
						  for(int k = 0; k < deltaYY; k++) {     
							if(*rpointer2)
							  *wpointer++ = o1 / *rpointer2;
							else 
							  *wpointer++ = 0;
							rpointer2++;
						  }                                   
						  wpointer  += hm - deltaYY;          
						  rpointer2 += h2 - deltaYY;
						}
					  }
					else 
					  {
						for(int h = 0; h < deltaXX; h++) {
						  for(int k = 0; k < deltaYY; k++) {     
							*wpointer++ = o1;
						  }                                   
						  wpointer  += hm - deltaYY;          
						}
					  }
					break;
				  case MAX:           ARITH2_OPER_MASK(max, COMMA); break;
				  case MIN:           ARITH2_OPER_MASK(min, COMMA); break;
				  case LESS_OR_EQ:    ARITH2_OPER_MASK(NOF, <=);    break;
				  case LESS:          ARITH2_OPER_MASK(NOF, <);     break;
				  case GREATER_OR_EQ: ARITH2_OPER_MASK(NOF, >=);    break;
				  case GREATER:       ARITH2_OPER_MASK(NOF, >);     break;
				  case EQUAL:         ARITH2_OPER_MASK(NOF, ==);    break;
				  case AND:           ARITH2_OPER_MASK(NOF, &);     break;
				  case OR:            ARITH2_OPER_MASK(NOF, |);     break;
				  case XOR:           ARITH2_OPER_MASK(NOF, ^);     break; 
				  default:
					UNPROTECT(1);
					return(R_NilValue);
				  }
				  break;
				case 7:
				  switch(opCode) {
				  case PLUS:          ARITH_OPER_MASK(NOF, +);     break;
				  case MINUS:         ARITH_OPER_MASK(NOF, -);     break;
				  case TIMES:         ARITH_OPER_MASK(NOF, *);     break;
				  case DIVIDE:        
					for(int h = 0; h < deltaXX; h++) {
					  for(int k = 0; k < deltaYY; k++) {     
						if(*rpointer2)
						  *wpointer++ = *rpointer1 / *rpointer2;
						else 
						  *wpointer++ = 0;
						rpointer1++;
						rpointer2++;
					  }                                   
					  wpointer  += hm - deltaYY;          
					  rpointer1 += h1 - deltaYY;
					  rpointer2 += h2 - deltaYY;
					}
					break;
				  case MAX:           ARITH_OPER_MASK(max, COMMA); break;
				  case MIN:           ARITH_OPER_MASK(min, COMMA); break;
				  case LESS_OR_EQ:    ARITH_OPER_MASK(NOF, <=);    break;
				  case LESS:          ARITH_OPER_MASK(NOF, <);     break;
				  case GREATER_OR_EQ: ARITH_OPER_MASK(NOF, >=);    break;
				  case GREATER:       ARITH_OPER_MASK(NOF, >);     break;
				  case EQUAL:         ARITH_OPER_MASK(NOF, ==);    break;
				  case AND:           ARITH_OPER_MASK(NOF, &);     break;
				  case OR:            ARITH_OPER_MASK(NOF, |);     break;
				  case XOR:           ARITH_OPER_MASK(NOF, ^);     break; 
				  default:
					UNPROTECT(1);
					return(R_NilValue);
				  }
				  break;
				}
			  }
		  }
	  }

	SET_SLOT(ani_r, install("outside"), ScalarInteger(oo));

	UNPROTECT(1);

	return(ani_r);
  }
  
  SEXP aniOpInt_im_im_im(SEXP animage_1,
						 SEXP ani_op,
						 SEXP animage_2,
						 SEXP animage_f)
  {
	SEXP ani_r;
    int	 XX[6], YY[6], REGIONS[5][5];
	int  x1, y1, w1, h1;
	int  x2, y2, w2, h2;
    int  xm, ym, wm, hm;
	int  o1, o2, om, of;

    // localize images 
	//
	x1 = GET_ANIMAGE_X(animage_1);
	y1 = GET_ANIMAGE_Y(animage_1);
	w1 = GET_ANIMAGE_W(animage_1);
	h1 = GET_ANIMAGE_H(animage_1);
	o1 = GET_ANIMAGE_OI(animage_1);

	x2 = GET_ANIMAGE_X(animage_2);
	y2 = GET_ANIMAGE_Y(animage_2);
	w2 = GET_ANIMAGE_W(animage_2);
	h2 = GET_ANIMAGE_H(animage_2);
	o2 = GET_ANIMAGE_OI(animage_2);

	// and mask (which now is determined by 
	// the filter image
	// 
	xm = GET_ANIMAGE_X(animage_f);
	ym = GET_ANIMAGE_Y(animage_f);
    wm = GET_ANIMAGE_W(animage_f);
    hm = GET_ANIMAGE_H(animage_f);
	om = GET_ANIMAGE_OI(animage_f);
	of = om;

	// fill in support structure for op macros
	//
	animage  filter;
	
	filter.idata = GET_ANIMAGE_DI(animage_f);
	filter.x     = xm;
	filter.y     = ym;
	filter.w     = wm;
	filter.h     = hm;
	filter.io    = om;

    // clearing REGIONS array 
	//
    for(int i = 0; i < 5; i++)
	  for(int j = 0; j < 5; j++)
	    REGIONS[i][j] = 0;
	    
    // compute table of delimiting coordinates of REGIONS 
	//
	XX[0] = xm;
	XX[1] = x1;
	XX[2] = x2;
	XX[3] = xm+wm;
	XX[4] = x1+w1;
	XX[5] = x2+w2;
     
	sorting(XX);

	YY[0] = ym;
	YY[1] = y1;
	YY[2] = y2;
	YY[3] = ym+hm;
	YY[4] = y1+h1;
	YY[5] = y2+h2;
	
	sorting(YY);

	/* 
	   REGIONS[i][j]  = 0 : out_of_matrix;
	   = 1 : image1 out of mask;
	   = 2 : image2 out of mask;
	   = 3 : image1 op image2 out of mask;
	   = 4 : mask;
	   = 5 : image1 in mask;
	   = 6 : image2 in mask;
	   = 7 : image1 op image2 in mask;
	*/
    
    for(int i = 0; i < 5; i++)
	  for(int j = 0; j < 5; j++)
	    if((XX[j] >= x1) && (XX[j] < x1+w1) &&
	       (YY[i] >= y1) && (YY[i] < y1+h1))
		  REGIONS[i][j] += 1;
	
    for(int i = 0; i < 5; i++)
	  for(int j = 0; j < 5; j++)
	    if((XX[j] >= x2) && (XX[j] < x2+w2) && 
	       (YY[i] >= y2) && (YY[i] < y2+h2))
		  REGIONS[i][j] += 2;

    for(int i = 0; i < 5; i++)
	  for(int j = 0; j < 5; j++)
	    if((XX[j] >= xm) && (XX[j] < xm+wm) && 
	       (YY[i] >= ym) && (YY[i] < ym+hm))
		  REGIONS[i][j] += 4;

	// compute outside of result
	//
	int opCode = statOpCode(CHAR(STRING_ELT(ani_op, 0)));
	int oo     = 0;

	switch (opCode)
	  {
	  case PLUS:          oo = o1 + o2; break;
	  case MINUS:         oo = o1 - o2; break;
	  case TIMES:         oo = o1 * o2; break;
	  case DIVIDE:        oo = (o2 > 0) ? o1 / o2 : 0; break;
	  case MAX:           oo = max(o1, o2); break;
	  case MIN:           oo = min(o1, o2); break;
	  case LESS_OR_EQ:    oo = (o1 <= o2) ? 1 : 0; break; 
	  case LESS:          oo = (o1 <  o2) ? 1 : 0; break; 
	  case GREATER_OR_EQ: oo = (o1 >= o2) ? 1 : 0; break; 
	  case GREATER:       oo = (o1 >= o2) ? 1 : 0; break; 
	  case EQUAL:         oo = (o1 == o2) ? 1 : 0; break; 
	  case AND:           oo = o1 & o2; break; 
	  case OR:            oo = o1 | o2; break;
	  case XOR:           oo = o1 ^ o2; break;
	  default:
		UNPROTECT(1);
		return(R_NilValue);
	  }

	// Create return animage
	//
	PROTECT(ani_r      = newAnimage(xm, ym, wm, hm, int(oo)));

	// and get the needed pointers to storage
	//
	int *wpointer_s  = GET_ANIMAGE_DI(ani_r);
	int *rpointer1_s = GET_ANIMAGE_DI(animage_1);
	int *rpointer2_s = GET_ANIMAGE_DI(animage_2) ;

	for(int i = 0; i < 5; i++) 
	  {
		int deltaYY = YY[i+1] - YY[i];
		int initY   = hm - deltaYY;

		for(int j = 0; j < 5; j++) 
		  {
			int deltaXX = XX[j+1] - XX[j];

			if((deltaXX > 0) && (deltaYY > 0))
			  {
				int *wpointer  = wpointer_s  + (XX[j] - xm)*hm + (YY[i]-ym);
				int *rpointer1 = rpointer1_s + (XX[j] - x1)*h1 + (YY[i]-y1);
				int *rpointer2 = rpointer2_s + (XX[j] - x2)*h2 + (YY[i]-y2);

				switch(REGIONS[i][j]) {
				case 0:
				case 1:
				case 2:
				case 3:
				  break;
				case 4:
				  for(int h = 0; h < deltaXX; h++) {
					for(int k = 0; k < deltaYY; k++) {     
					  if(get_pixel(filter, YY[i] + k, XX[j] + h))
						*wpointer++ = oo;
					  else
						*wpointer++ = om;
					}                                   
					wpointer  += hm - deltaYY;          
				  }
				  break;
				case 5: 
				  switch(opCode) {
				  case PLUS:          ARITH1_OPER_FILT(NOF, +);     break;
				  case MINUS:         ARITH1_OPER_FILT(NOF, -);     break;
				  case TIMES:         ARITH1_OPER_FILT(NOF, *);     break;
				  case DIVIDE:        
					if(o2)
					  {
						for(int h = 0; h < deltaXX; h++) {
						  for(int k = 0; k < deltaYY; k++) {     
							if(get_pixel(filter, YY[i] + k, XX[j] + h))
							  {
								*wpointer++ = *rpointer1 / o2;
								rpointer1++;
							  }
							else 
							  {
								*wpointer++ = om;
							  }
						  }                                   
						  wpointer  += hm - deltaYY;    
						  rpointer1 += h1 - deltaYY;
						}
					  }
					else 
					  {
						for(int h = 0; h < deltaXX; h++) {
						  for(int k = 0; k < deltaYY; k++) {     
							*wpointer++ = 0;
						  }                                   
						  wpointer  += hm - deltaYY;          
						}
					  }
					break;
				  case MAX:           ARITH1_OPER_FILT(max, COMMA); break;
				  case MIN:           ARITH1_OPER_FILT(min, COMMA); break;
				  case LESS_OR_EQ:    ARITH1_OPER_FILT(NOF, <=);    break;
				  case LESS:          ARITH1_OPER_FILT(NOF, <);     break;
				  case GREATER_OR_EQ: ARITH1_OPER_FILT(NOF, >=);    break;
				  case GREATER:       ARITH1_OPER_FILT(NOF, >);     break;
				  case EQUAL:         ARITH1_OPER_FILT(NOF, ==);    break;
				  case AND:           ARITH1_OPER_FILT(NOF, &);     break;
				  case OR:            ARITH1_OPER_FILT(NOF, |);     break;
				  case XOR:           ARITH1_OPER_FILT(NOF, ^);     break; 
				  default:
					UNPROTECT(1);
					return(R_NilValue);
				  }
				  break;
				case 6:
				  switch(opCode) {
				  case PLUS:          ARITH2_OPER_FILT(NOF, +);     break;
				  case MINUS:         ARITH2_OPER_FILT(NOF, -);     break;
				  case TIMES:         ARITH2_OPER_FILT(NOF, *);     break;
				  case DIVIDE:        
					if(o1)
					  {
						for(int h = 0; h < deltaXX; h++) {
						  for(int k = 0; k < deltaYY; k++) {     
							if(get_pixel(filter, YY[i] + k, XX[j] + h))
							  {
								if(*rpointer2) 
								  *wpointer = o1 / *rpointer2;
								else
								  *wpointer = 0;
							  }
							else
							  *wpointer = of;

							wpointer++;
							rpointer2++;
						  }                                   
						  wpointer  += hm - deltaYY;          
						  rpointer2 += h2 - deltaYY;
						}
					  }
					else 
					  {
						for(int h = 0; h < deltaXX; h++) {
						  for(int k = 0; k < deltaYY; k++) {     
							*wpointer++ = o1;
						  }                                   
						  wpointer  += hm - deltaYY;          
						}
					  }
					break;
				  case MAX:           ARITH2_OPER_FILT(max, COMMA); break;
				  case MIN:           ARITH2_OPER_FILT(min, COMMA); break;
				  case LESS_OR_EQ:    ARITH2_OPER_FILT(NOF, <=);    break;
				  case LESS:          ARITH2_OPER_FILT(NOF, <);     break;
				  case GREATER_OR_EQ: ARITH2_OPER_FILT(NOF, >=);    break;
				  case GREATER:       ARITH2_OPER_FILT(NOF, >);     break;
				  case EQUAL:         ARITH2_OPER_FILT(NOF, ==);    break;
				  case AND:           ARITH2_OPER_FILT(NOF, &);     break;
				  case OR:            ARITH2_OPER_FILT(NOF, |);     break;
				  case XOR:           ARITH2_OPER_FILT(NOF, ^);     break; 
				  default:
					UNPROTECT(1);
					return(R_NilValue);
				  }
				  break;
				case 7:
				  switch(opCode) {
				  case PLUS:          ARITH_OPER_FILT(NOF, +);     break;
				  case MINUS:         ARITH_OPER_FILT(NOF, -);     break;
				  case TIMES:         ARITH_OPER_FILT(NOF, *);     break;
				  case DIVIDE:        
					for(int h = 0; h < deltaXX; h++) {
					  for(int k = 0; k < deltaYY; k++) {     
						if(get_pixel(filter, YY[i] + k, XX[j] + h))
						  {
							if(*rpointer2)
							  *wpointer = *rpointer1 / *rpointer2;
							else 
							  *wpointer = 0;
						  }
						else
						  *wpointer = of;

						wpointer++;
						rpointer1++;
						rpointer2++;
					  }                                   
					  wpointer  += hm - deltaYY;          
					  rpointer1 += h1 - deltaYY;
					  rpointer2 += h2 - deltaYY;
					}
					break;
				  case MAX:           ARITH_OPER_FILT(max, COMMA); break;
				  case MIN:           ARITH_OPER_FILT(min, COMMA); break;
				  case LESS_OR_EQ:    ARITH_OPER_FILT(NOF, <=);    break;
				  case LESS:          ARITH_OPER_FILT(NOF, <);     break;
				  case GREATER_OR_EQ: ARITH_OPER_FILT(NOF, >=);    break;
				  case GREATER:       ARITH_OPER_FILT(NOF, >);     break;
				  case EQUAL:         ARITH_OPER_FILT(NOF, ==);    break;
				  case AND:           ARITH_OPER_FILT(NOF, &);     break;
				  case OR:            ARITH_OPER_FILT(NOF, |);     break;
				  case XOR:           ARITH_OPER_FILT(NOF, ^);     break; 
				  default:
					UNPROTECT(1);
					return(R_NilValue);
				  }
				  break;
				}
			  }
		  }
	  }

	UNPROTECT(1);

	return(ani_r);
  }

  SEXP aniGetInt_mk(SEXP animage_1,
					SEXP animask_f)
  {
	SEXP ani_r;
    int	 XX[4], YY[4], REGIONS[3][3];
	int  x1, y1, w1, h1;
    int  xm, ym, wm, hm;
	int  o1;

    // localize images 
	//
	x1 = GET_ANIMAGE_X(animage_1);
	y1 = GET_ANIMAGE_Y(animage_1);
	w1 = GET_ANIMAGE_W(animage_1);
	h1 = GET_ANIMAGE_H(animage_1);
	o1 = GET_ANIMAGE_OI(animage_1);

	// and mask (which now is determined by 
	// the filter image)
	// 
	xm = GET_ANIMASK_X(animask_f);
	ym = GET_ANIMASK_Y(animask_f);
    wm = GET_ANIMASK_W(animask_f);
    hm = GET_ANIMASK_H(animask_f);

    // clearing REGIONS array 
	//
    for(int i = 0; i < 3; i++)
	  for(int j = 0; j < 3; j++)
	    REGIONS[i][j] = 0;
	    
    // compute table of delimiting coordinates of REGIONS 
	//
    XX[0]   = min(x1, xm);
    XX[1]   = min(max(x1, xm), min(x1+w1, xm+wm));
    XX[2]   = max(max(x1, xm), min(x1+w1, xm+wm));
    XX[3]   = max(x1+w1, xm+wm);

    YY[0]   = min(y1, ym);
    YY[1]   = min(max(y1, ym), min(y1+h1, ym+hm));
    YY[2]   = max(max(y1, ym), min(y1+h1, ym+hm));
    YY[3]   = max(y1+h1, ym+hm);

	/* 
	   REGIONS[i][j]	= 0 : out_of_matrix out of mask;
	   = 1 : image out of mask;
	   = 2 : out_of_matrix in mask;
	   = 3 : image in mask;
	*/

    for(int i = 0; i < 3; i++)
	  for(int j = 0; j < 3; j++)
	    if((XX[j] >= x1) && (XX[j] < x1+w1) && 
	       (YY[i] >= y1) && (YY[i] < y1+h1))
		  REGIONS[i][j] += 1;

    for(int i = 0; i < 3; i++)
	  for(int j = 0; j < 3; j++)
	    if((XX[j] >= xm) && (XX[j] < xm+wm) && 
	       (YY[i] >= ym) && (YY[i] < ym+hm))
		  REGIONS[i][j] += 2;

	// Create return animage
	//
	PROTECT(ani_r      = newAnimage(xm, ym, wm, hm, int(o1)));

	// and get the needed pointers to storage
	//
	int *wpointer_s  = GET_ANIMAGE_DI(ani_r);
	int *rpointer1_s = GET_ANIMAGE_DI(animage_1);

	for(int i = 0; i < 3; i++) 
	  {
		int deltaYY = YY[i+1] - YY[i];
		int initY   = hm - deltaYY;

		for(int j = 0; j < 3; j++) 
		  {
			int deltaXX = XX[j+1] - XX[j];

			if((deltaXX > 0) && (deltaYY > 0))
			  {
				int *wpointer  = wpointer_s  + (XX[j] - xm)*hm + (YY[i]-ym);
				int *rpointer1 = rpointer1_s + (XX[j] - x1)*h1 + (YY[i]-y1);

				switch(REGIONS[i][j]) {
				case 0:
				case 1:
				  break;
				case 2:
				  for(int h = 0; h < deltaXX; h++)  {    
					for(int k = 0; k < deltaYY; k++)  {
					  *wpointer++ = o1;
					}                               
					wpointer  += initY;                
				  }
				  break;
				case 3:
				  for(int h = 0; h < deltaXX; h++)  {    
					for(int k = 0; k < deltaYY; k++)  {
					  *wpointer++ = *rpointer1++;
					}                                  
					wpointer  += initY;                
					rpointer1 += h1 - deltaYY;         
				  }
				  break;
				}
			  }
		  }
	  }

	UNPROTECT(1);

	return(ani_r);
  }

  SEXP aniGetInt_px(SEXP animage_1,
					SEXP animage_f)
  {
	SEXP ani_r;
	int  x1, y1, w1, h1;
	int  x, y, vv;
	

    // localize image
	//
	x1 = GET_ANIMAGE_X(animage_1);
	y1 = GET_ANIMAGE_Y(animage_1);
	w1 = GET_ANIMAGE_W(animage_1);
	h1 = GET_ANIMAGE_H(animage_1);
	vv = GET_ANIMAGE_OI(animage_1);

	y  = INTEGER(animage_f)[0];
	x  = INTEGER(animage_f)[1];

	if((x1 <= x) && (x < x1 + w1) && (y1 <= y) && (y < y1 + h1))
	  {
		vv = GET_ANIMAGE_DI(animage_1)[(x-x1)*h1+(y-y1)];
	  }

	PROTECT(ani_r = ScalarInteger(vv));

	UNPROTECT(1);

	return(ani_r);
  }

  SEXP aniSetInt_px(SEXP animage_1,
					SEXP animage_f,
					SEXP anivalue)
  {
	int  x1, y1, w1, h1;
	int  x, y;
	

    // localize image
	//
	x1 = GET_ANIMAGE_X(animage_1);
	y1 = GET_ANIMAGE_Y(animage_1);
	w1 = GET_ANIMAGE_W(animage_1);
	h1 = GET_ANIMAGE_H(animage_1);

	y  = INTEGER(animage_f)[0];
	x  = INTEGER(animage_f)[1];

	if((x1 <= x) && (x < x1 + w1) && (y1 <= y) && (y < y1 + h1))
	  {
		GET_ANIMAGE_DI(animage_1)[(x-x1)*h1+(y-y1)] = INTEGER(anivalue)[0];
	  }

	return(anivalue);
  }

  SEXP aniGetInt_im(SEXP animage_1,
					SEXP animage_f)
  {
	SEXP ani_r;
    int	 XX[4], YY[4], REGIONS[3][3];
	int  x1, y1, w1, h1;
    int  xm, ym, wm, hm;
	int  o1, om, of;

    // localize images 
	//
	x1 = GET_ANIMAGE_X(animage_1);
	y1 = GET_ANIMAGE_Y(animage_1);
	w1 = GET_ANIMAGE_W(animage_1);
	h1 = GET_ANIMAGE_H(animage_1);
	o1 = GET_ANIMAGE_OI(animage_1);

	// and mask (which now is determined by 
	// the filter image
	// 
	xm = GET_ANIMAGE_X(animage_f);
	ym = GET_ANIMAGE_Y(animage_f);
    wm = GET_ANIMAGE_W(animage_f);
    hm = GET_ANIMAGE_H(animage_f);
	om = GET_ANIMAGE_OI(animage_f);
	of = o1;

	// fill in support structure for op macros
	//
	animage  filter;
	
	filter.idata = GET_ANIMAGE_DI(animage_f);
	filter.x     = xm;
	filter.y     = ym;
	filter.w     = wm;
	filter.h     = hm;
	filter.io    = om;

    // clearing REGIONS array 
	//
    for(int i = 0; i < 3; i++)
	  for(int j = 0; j < 3; j++)
	    REGIONS[i][j] = 0;
	    
    // compute table of delimiting coordinates of REGIONS 
	//
    XX[0]   = min(x1, xm);
    XX[1]   = min(max(x1, xm), min(x1+w1, xm+wm));
    XX[2]   = max(max(x1, xm), min(x1+w1, xm+wm));
    XX[3]   = max(x1+w1, xm+wm);

    YY[0]   = min(y1, ym);
    YY[1]   = min(max(y1, ym), min(y1+h1, ym+hm));
    YY[2]   = max(max(y1, ym), min(y1+h1, ym+hm));
    YY[3]   = max(y1+h1, ym+hm);

	/* 
	   REGIONS[i][j]	= 0 : out_of_matrix out of filter;
	   = 1 : image out of filter;
	   = 2 : out_of_matrix in filter;
	   = 3 : image in filter;
	*/
    
    for(int i = 0; i < 3; i++)
	  for(int j = 0; j < 3; j++)
	    if((XX[j] >= x1) && (XX[j] < x1+w1) && 
	       (YY[i] >= y1) && (YY[i] < y1+h1))
		  REGIONS[i][j] += 1;

    for(int i = 0; i < 3; i++)
	  for(int j = 0; j < 3; j++)
	    if((XX[j] >= xm) && (XX[j] < xm+wm) && 
	       (YY[i] >= ym) && (YY[i] < ym+hm))
		  REGIONS[i][j] += 2;

	// Create return animage
	//
	PROTECT(ani_r      = newAnimage(xm, ym, wm, hm, int(o1)));

	// and get the needed pointers to storage
	//
	int *wpointer_s  = GET_ANIMAGE_DI(ani_r);
	int *rpointer1_s = GET_ANIMAGE_DI(animage_1);

	for(int i = 0; i < 3; i++) 
	  {
		int deltaYY = YY[i+1] - YY[i];
		int initY   = hm - deltaYY;

		for(int j = 0; j < 3; j++) 
		  {
			int deltaXX = XX[j+1] - XX[j];

			if((deltaXX > 0) && (deltaYY > 0))
			  {
				int *wpointer  = wpointer_s  + (XX[j] - xm)*hm + (YY[i]-ym);
				int *rpointer1 = rpointer1_s + (XX[j] - x1)*h1 + (YY[i]-y1);

				switch(REGIONS[i][j]) {
				case 0:
				case 1:
				  break;
				case 2:
				  for(int h = 0; h < deltaXX; h++)  {    
					for(int k = 0; k < deltaYY; k++)  {
					  if(get_pixel(filter, YY[i] + k, XX[j] + h))         
						*wpointer++ = o1;
					  else 
						*wpointer++ = of;            
					}                               
					wpointer  += initY;                
				  }
				  break;
				case 3:
				  for(int h = 0; h < deltaXX; h++)  {    
					for(int k = 0; k < deltaYY; k++)  {
					  if(get_pixel(filter, YY[i] + k, XX[j] + h))         
						*wpointer++ = *rpointer1++;
					  else {
						*wpointer++ = of;            
						rpointer1++;                
					  }                                
					}                                  
					wpointer  += initY;                
					rpointer1 += h1 - deltaYY;         
				  }
				  break;
				}
			  }
		  }
	  }

	UNPROTECT(1);

	return(ani_r);
  }

  SEXP aniOpInt_im_ss(SEXP animage_1,
					  SEXP ani_op,
					  SEXP ani_2)
  {
	SEXP ani_r = R_NilValue;

	int x1 = GET_ANIMAGE_X(animage_1);
	int y1 = GET_ANIMAGE_Y(animage_1);
    int w1 = GET_ANIMAGE_W(animage_1);
    int h1 = GET_ANIMAGE_H(animage_1);
	int o1 = GET_ANIMAGE_OI(animage_1);
	
	// get the value of the 'immediate' parameter
	//
	int vv     = INTEGER(ani_2)[0];
 	int opCode = statOpCode(CHAR(STRING_ELT(ani_op, 0)));
	int oo     = 0;

	switch (opCode)
	  {
	  case PLUS:          oo = o1 + vv; break;
	  case MINUS:         oo = o1 - vv; break;
	  case TIMES:         oo = o1 * vv; break;
	  case DIVIDE:        oo = (vv > 0) ? o1 / vv : 0; break;
	  case MAX:           oo = max(o1, vv); break;
	  case MIN:           oo = min(o1, vv); break;
	  case LESS_OR_EQ:    oo = (o1 <= vv) ? 1 : 0; break; 
	  case LESS:          oo = (o1 <  vv) ? 1 : 0; break; 
	  case GREATER_OR_EQ: oo = (o1 >= vv) ? 1 : 0; break; 
	  case GREATER:       oo = (o1 >= vv) ? 1 : 0; break; 
	  case EQUAL:         oo = (o1 == vv) ? 1 : 0; break; 
	  case AND:           oo = o1 &  vv; break; 
	  case OR:            oo = o1 |  vv; break;
	  case XOR:           oo = o1 ^  vv; break;
	  case SHIFT_LEFT:    oo = o1 << vv; break;
	  case SHIFT_RIGHT:   oo = o1 >> vv; break;
	  case NOT:           oo = (o1 ^ vv) & vv; break;
	  case SET:           oo = vv; break;
	  default:
		UNPROTECT(1);
		return(R_NilValue);
	  }

	// create return animage
	//
	PROTECT(ani_r      = newAnimage(x1, y1, w1, h1, int(oo)));

	// and get the needed pointers to storage
	//
	int *wpointer = GET_ANIMAGE_DI(ani_r);
	int *rpointer = GET_ANIMAGE_DI(animage_1);

	int  i = w1 * h1;

	switch (opCode)
	  {
	  case PLUS:          ARITH_OPER_IMM(NOF, +);     break;
	  case MINUS:         ARITH_OPER_IMM(NOF, -);     break;
	  case TIMES:         ARITH_OPER_IMM(NOF, *);     break;
	  case MAX:           ARITH_OPER_IMM(max, COMMA); break;
	  case MIN:           ARITH_OPER_IMM(min, COMMA); break;
	  case LESS_OR_EQ:    ARITH_OPER_IMM(NOF, <=);    break; 
	  case LESS:          ARITH_OPER_IMM(NOF,  <);    break; 
	  case GREATER_OR_EQ: ARITH_OPER_IMM(NOF, >=);    break; 
	  case GREATER:       ARITH_OPER_IMM(NOF,  >);    break; 
	  case EQUAL:         ARITH_OPER_IMM(NOF, ==);    break; 
	  case AND:           ARITH_OPER_IMM(NOF,  &);    break; 
	  case OR:            ARITH_OPER_IMM(NOF,  |);    break;
	  case XOR:           ARITH_OPER_IMM(NOF,  ^);    break;
	  case SHIFT_LEFT:    ARITH_OPER_IMM(NOF, <<);    break;
	  case SHIFT_RIGHT:   ARITH_OPER_IMM(NOF, >>);    break;
	  case NOT:           
		while(i--) {                        
		  *wpointer++ = (*rpointer ^ vv) & vv; 
		  rpointer++;                      
		}
		break;
	  case SET:           
		while(i--) {                        
		  *wpointer++ = vv; 
		  rpointer++;                      
		}
		break;
	  case DIVIDE:        
		if(vv)
		  {
			while(i--) {                        
			  *wpointer++ = *rpointer / vv;
			  rpointer++;                      
			}
		  }
		else 
		  {
			// we should generate an error
		  }
		break;
		
	  default:
		UNPROTECT(1);
		return(R_NilValue);
	  }

	UNPROTECT(1);
	return(ani_r);
  }
}
