#ifndef CONVOLUTION_H
#define CONVOLUTION_H

#include <arrayTools/Array.h>

#include <arrayTools/namespace_header.h>


namespace _ARRAYTOOLS_NAMESPACE
{

  //! Return the convolution of A by K 
  //! The outer frame (whose size is determined by
  //! that of the convolution kernel K) is filled with 0
  //!
  array<float>& Convolution(array<float>& A, array<float>& K);

  //! Return the convolution of A by K 
  //! Only where F == TRUE
  array<float>& Convolution(array<float>& A, array<float>& K, array<u_char> &F, float fillValu);



  //! Return the convolution of A by K 
  //! The outer frame (whose size is determined by
  //! that of the convolution kernel K) is filled with 0
  template<class T1, class T2, class T3> void Convolution(array<T1>& A, array<T2>& K, array<T3> O)
    {
    

      int		 x, y, wx, wy;
      int		 i, j;
      int           x0, x1;
      int           y0, y1;
      double	 v;

      T1         **_A; 
      T1         * _a;   
      T2         **_K; 
      T2         * ptr; 
   
      wx 		= K.cols() / 2;
      wy 		= K.rows() / 2;

      x0           = A.first_col_index() + wx;
      x1           = A.last_col_index()  - wx;
      y0           = A.first_row_index() + wy;
      y1           = A.last_row_index()  - wy;
   
      _A           = A.storage();
      _K           = K.storage();
   
      O 		= new array<T3>(A.first_row_index(),
					A.last_row_index(),
					A.first_col_index(),
					A.last_col_index());
   
      for(y = y0; y <= y1; y++)
	{
	  for(x = x0; x <= x1; x++)
	    {
	      v = 0.0;
	      for(j = -wy; j <= wy; j++)
		{
		  ptr = _K[j+wy];
		  _a  = &(_A[y+j][x-wx]);
		  
		  for(i = -wx;  i <= wx; i++)
		    {
		      v += (double)(*_a++) * (double)(*ptr++);
		    }
		}
	      (*O)[y][x] = (T3) v;
	    }
	}

      DeleteIfTemporary(A);
      DeleteIfTemporary(K);
   
   
    }

}


#endif
