#
#
#
tm.bhatNayarCorrelation <- function(img,
                                    template,
                                    alpha = 0.5)
{
  #
  if(!is.animage(img))
    stop("<img> must be an animage");
  #
  # The current implementation only works on
  # integer images with values in the range
  # [0,255] so we enforce the constraint with a linear
  # rescaling.
  #
  img <- ia.integerAnimage(ia.stretchRange(img, dstRange=c(0,255)));
  #
  if(!is.animage(template))
    stop("<template> must be an animage");
  #
  template <- ia.integerAnimage(ia.stretchRange(template, dstRange=c(0,255)));
  #
  # alpha controls the blending between two versions of the
  # Bhat-Nayar distance: it must lie in the interval [0,1]
  #
  if((alpha < 0) || (1 < alpha)) 
    stop("distance blending factor <alpha> must be in [0,1]");
  
  res <- .Call("tmBhatNayarCorrelation",
               img,
               template,
               alpha,
               PACKAGE="TeMa")
  
  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res[[1]]@history  <- updatedHistory
    res[[1]]@document <- TRUE
  } else {
    res[[1]]@history  <- call("<undef>")
    res[[1]]@document <- FALSE
  }

  res
}
