#
#
#
tm.bitPack <- function(X) {
  x <- 0L

  for(i in X) 
    if(i != 0L)
      x <- (x * 2L) + 1L
    else
      x <- (x * 2L)

  x
}

tm.bitUnpack <- function(x, width = 8) {
  b <- c()

  for(i in 1:width)
    {
      b <- c(x %% 2L, b)
      x <- x %/% 2L 
    }

  b
}

tm.bitAnd <- function(X, Y)
{
  if(!(is.vector(X)) || !(is.integer(X)))
    stop("X must be a INTEGER vector");

  if(!(is.vector(Y)) || !(is.integer(Y)))
    stop("Y must be a INTEGER vector");

  res <- .Call("tmBitAnd",
               X,
               Y,
               PACKAGE="TeMa")

  res
}

tm.bitOr <- function(X, Y)
{
  if(!(is.vector(X)) || !(is.integer(X)))
    stop("X must be a INTEGER vector");

  if(!(is.vector(Y)) || !(is.integer(Y)))
    stop("Y must be a INTEGER vector");

  res <- .Call("tmBitOr",
               X,
               Y,
               PACKAGE="TeMa")

  res
}

tm.bitXor <- function(X, Y)
{
  if(!(is.vector(X)) || !(is.integer(X)))
    stop("X must be a INTEGER vector");

  if(!(is.vector(Y)) || !(is.integer(Y)))
    stop("Y must be a INTEGER vector");

  res <- .Call("tmBitXor",
               X,
               Y,
               PACKAGE="TeMa")

  res
}

tm.bitNot <- function(X)
{
  if(!(is.vector(X)) || !(is.integer(X)))
    stop("X must be a INTEGER vector");

  res <- .Call("tmBitNot",
               X,
               PACKAGE="TeMa")

  res
}

tm.bitSum <- function(X)
{
  
  if(!(is.vector(X)) || !(is.integer(X)))
    stop("X must be a INTEGER vector");
  
  res <- .Call("tmBitSum",
               X,
               PACKAGE="TeMa")

  res
}

tm.hamming <- function(X, Y, width=6)
  {
    if(is.animage(X) && is.animage(Y))
      sum(tm.hamming(as.vector(X@data), as.vector(Y@data), width=width)) / ia.size(X)
    else
      tm.bitSum(tm.bitXor(X, Y)) / width
  }

tm.tanimoto <- function(X, Y)
  {
    if(is.animage(X) && is.animage(Y))
      {
        sum(tm.tanimoto(as.vector(X@data), as.vector(Y@data))) / ia.size(X)
      }
    else
      {
        num <- tm.bitSum(tm.bitAnd(X, Y))
        den <- tm.bitSum(tm.bitAnd(X, X)) + tm.bitSum(tm.bitAnd(Y, Y)) - num

        n   <- length(X)
        res <- rep(1L, n)

        for(i in 1:n)
          if(den[i] > 0L)
            res[i] <- 1 - num[i]/den[i]
    
        res
      }
  }

tm.dixonKoehler <- function(X, Y, width=6)
  {
    if(is.animage(X) && is.animage(Y))
      sum(tm.dixonKoehler(as.vector(X@data), as.vector(Y@data), width=width)) / ia.size(X)
    else
      tm.tanimoto(X,Y) * tm.hamming(X, Y,width=width)
  }
