#
# Get the ppm depth map and unwrap depth information from the RGB channels
# into a single channel image
#
# The default value for metric is 10 because distances from POVRay are expressed in cm
# but all lens computations we perform require mm
#

tm.getDepth <- function(filename, depthScale = 1, metric = 10)
  {
    #
    depthInfo <- read.pnm(filename);
    #
    res <- as.animage(((getChannels(depthInfo, colors=c("red"))   * 256 +
                        getChannels(depthInfo, colors=c("green")) * 256 * 256 +
                        getChannels(depthInfo, colors=c("blue"))  * 256 * 256 * 256)/depthScale)*metric)
    #
    return(as.animage(res))
    #   
  }



#
# Apply focus blur as a post processing step
# The image must be provided as a list of
# channels (a list of 1 channel is legal)
#
# focalLength and focalDistance must be expressed in mm
# focalDistance must be greater than focalLength
#
# scale is used to map the size of the circle of confusion onto
# the pixels
#
# (see also http://www.dofmaster.com/dofjs.html)
#

tm.focusBlur <- function(channels,
                         depthMap,
                         focalLength,
                         fStop,
                         focalDistance,
                         scale     = (1201/36.0), # 1201 pixels over a 36 mm wide sensor
                         maxRadius = 30)
{
  #
  if(!ia.congruent(channels, depthMap)) {
    stop("non congruent image channels and depth map")
  }
  #
  if(focalDistance <= focalLength) {
    stop("Focal distance must be greater than focal length")
  }
  #
  res <- .Call("tmFocusBlur2D",
               channels,
               depthMap,
               focalLength,
               fStop,
               focalDistance,
               scale,
               maxRadius,
               PACKAGE="TeMa")
  #
  return(res)
  #
}
