#
# Transform degrees into radiants
#
tm.rad <- function(deg) {
  return (0.017453293 * deg)
}


# tm.houghLineCell supersample a Hough space cell, drawing
# the corresponding image space template for each supersampled
# point. The result is the image space template corresponding
# to a finite extent Hough space cell.
#
#
tm.houghLineCell <- function(halfSide,
                             r,   # radial cell coordinate
                             t,   # angular cell coordinate
                             dr,  # radial cell size
                             dt,  # angular cell size
                             n) { # linear cell supersampling factor
  #  
  for(j in 0:(n-1)) {
    #
    r1 <- r + (j*dr)/n
    #
    for(i in 0:(n-1)) {
      #
      t1    <- t + (i*dt)/n
      #
      x0    <- c(r1/cos(t1),0)
      x1    <- c(0, r1/sin(t1))
      #
      m     <- (x1[2]-x1[1]) / (x0[2]-x0[1])
      q     <- -m * x0[1] + x0[2]
      #
      x0[1] <- - halfSide
      x0[2] <- x0[1]*m + q
      #
      x1[1] <-   halfSide
      x1[2] <- x1[1]*m + q
      #
      lines(c(x0[1],x1[1]),c(x0[2],x1[2]))
    }
  }
  #
}

#
# Clear image space: to be invoked before
# projecting Hough space cells back into image space
#
tm.houghImageSpaceClean <- function(halfSide = 512) {
  plot(-2*halfSide,
       xlim=c(-halfSide,halfSide),
       ylim=c(-halfSide,halfSide),
       xlab="x", ylab="y", main="Image space Hough templates")

  abline(h=0)
  abline(v=0)
}
