#
# \litcode{Interlaced image generation}
#
# Interlaced images are built by juxtaposition of two
# fields, each providing a partial image representation.
# This function simulates the generation of a full resolution
# image from two different fields with user selectable
# field {\em distance} specified by \code{delta}.
#
tm.interlaceImage <- function(img, delta = 0L) {
  # We get the field composed by the odd lines
  oddField  <- tm.imageField(img, odd = TRUE)
  # and the one by the even lines
  evenField <- tm.imageField(img, odd = FALSE)
  #
  # We can simulate different time lapse factors
  # by changing the value of \code{delta} and
  # translating one of the fields accordingly:
  #
  if(delta != 0) {
    # The full resolution image is simply obtained
    # by summing the two fields:
    #
    img <- ia.get(ia.add(oddField,
                         ia.left(evenField, delta)),
                  ia.mask(img))
  } else {
    # A value \code{delta=0} corresponds to a static
    # scene and stationary camera and is equivalent
    # to the output of a progressive sensor:
    img
  }
}
