#
# Lanczos kernel coefficient computation (vectorized)
# A value of support = 2 is appropriate for interpolation,
# as higher values tend to provide ringing (they could be useful
# to preprocess images for print).
#
#

tm.Lanczos <- function(r, support = 2)
{
  if(is.vector(r) && (length(r) > 1)) {
    sapply(r, tm.Lanczos, support)
  } else {
    if(r == 0) {
      return(1)
    } else if(abs(r) < support) {
      return(support*sin(pi*r)*sin(pi*r/support)/(pi*pi*r*r))
    } else {
      return(0)
    }
  }
}

tm.lanczosKernel <- function(factor = 2.0, support = 2, trim = TRUE)
  {
    if(is.logical(trim))
      actualSupport <- as.integer(support * factor + 0.5)
    else
      actualSupport <- trim
    
    kernel        <- array(0, c(1, 2*actualSupport+1))
      
    i = 1;
    for(x in -actualSupport:actualSupport) {
      kernel[1, i] <- tm.Lanczos(x/factor,support)
      i <- i+1
    }

    kernel
  }
