require(TeMa)
#
# User control demo progress
#
opar  <- par(ask = dev.interactive(orNone = TRUE))
#
tiffdir          <- file.path(system.file(package = "TeMa"), "sampleimages/cameraSimulation")
#
i_ortho          <- ia.scale(ia.readAnimage(file.path(tiffdir, "ortho_iso12233_large.tif")), maxValue = 255)
i_orthoRed       <- i_ortho[[1]]
i_orthoRedDetail <- ia.get(i_orthoRed, animask(0,0,1201,256))
#
i_persp          <- ia.scale(ia.readAnimage(file.path(tiffdir, "persp_iso12233_large.tif")), maxValue = 255)
i_perspRed       <- i_persp[[1]]
i_perspRedDetail <- ia.get(i_perspRed, animask(0,0,1201,256))
#
plot(pixmapGrey(i_perspRedDetail@data),
     main= "Detail of nuts on ISO 12233 resolution chart")
#
k_diffraction_f2 <- tm.diffractionKernel(focalLength= 8.9e-3,
                                         fStop=       2,
                                         pixelSize=   (8.8/3264)/1000,
                                         superSampling=3)
#
persp(k_diffraction_f2,
      xlab="x", ylab="y", zlab="Airy function",
      main="Coolpix 8800 diffraction kernel for f2 (3x sampling)")
#
k_diffraction_f5 <- tm.diffractionKernel(focalLength= 8.9e-3,
                                         fStop=       5,
                                         pixelSize=   (8.8/3264)/1000,
                                         superSampling=3)
#
persp(k_diffraction_f5,
      xlab="x", ylab="y", zlab="Airy function",
      main="Coolpix 8800 diffraction kernel for f5 (3x sampling)")
#
k_diffraction_f16 <- tm.diffractionKernel(focalLength= 8.9e-3,
                                         fStop=       16,
                                         pixelSize=   (8.8/3264)/1000,
                                         superSampling=3)
#
persp(k_diffraction_f16,
      xlab="x", ylab="y", zlab="Airy function",
      main="Coolpix 8800 diffraction kernel for f16 (3x sampling)")
#
k_diffraction_f16_D70s <- tm.diffractionKernel(focalLength=50e-3,
                                               fStop=16,
                                               pixelSize=(23.7/3008)/1000,
                                               superSampling = 3)
#
persp(k_diffraction_f16_D70s,
      xlab="x", ylab="y", zlab="Airy function",
      main="D70s diffraction kernel for f16 (3x sampling)")
#
i_perspRedDetail_f2 <- ia.convolution(i_perspRedDetail,
                                       tm.diffractionKernel(focalLength= 8.9e-3,
                                                            fStop=       2,
                                                            pixelSize=   (8.8/3264)/1000))
#
i_perspRedDetail_f5 <- ia.convolution(i_perspRedDetail,
                                       tm.diffractionKernel(focalLength= 8.9e-3,
                                                            fStop=       5,
                                                            pixelSize=   (8.8/3264)/1000))
#
i_perspRedDetail_f16 <- ia.convolution(i_perspRedDetail,
                                       tm.diffractionKernel(focalLength= 8.9e-3,
                                                            fStop=       16,
                                                            pixelSize=   (8.8/3264)/1000))
#
plot(pixmapGrey(i_perspRedDetail_f16@data),
     main= "Coolpix 8800 blurring effect of diffraction due to a small (f16) aperture")
#
i_perspRedDetail_f16 <- ia.convolution(i_perspRedDetail,
                                       tm.diffractionKernel(focalLength= 50e-3,
                                                            fStop=       16,
                                                            pixelSize=   (23.7/3008)/1000))
#
# Images are affected by the quantum nature of light: photon noise. Note
# that our synthetic images use the sRGB color space so we first go back
# to a linear space, add the noise, and map again to sRGB. 
#
i_perspRedDetail_f16_iso1600 <-
  ia.apply(i_perspRedDetail_f16,
           function(x, y, z)
              {
               tm.linearToSrgb(
                 tm.photonNoise(tm.srgbToLinear(z/255),
                                gammaCorrection = 1,
                                maxValue        = 1,
                                fullWell        = 48800,
                                fullWellIso     = 100,
                                iso             = 1600)) * 255})
#
plot(pixmapGrey(i_perspRedDetail_f16_iso1600@data),
     main= "Nikon D70 simulation of ISO 1600 at f16")
#
# Real lenses vignetting
#
distagonVignetteMap <- tm.computeVignettingMap(width=1201, height = 801,
                                               curve = distagonVignette)
#
plot(pixmapGrey(tm.vignetteImage(i_perspRed, distagonVignetteMap)@data),
     main = "Vignetting effect of Distagon f/2.8 - 21mm")
#
# and distortion 
#
distagonDistortionMaps <- tm.computeDistortionMaps(width = 1201, height = 801, curve = distagonDistortion)
#
i_perspRed_distorted <- tm.distortImage(i_perspRed, distagonDistortionMaps)
#
plot(pixmapGrey(i_perspRed_distorted@data),
     main = "Distortion effect of Distagon f/2.8 - 21mm")
#
# Now with some examples based on color, illustrating the effect of non full RGB imaging
#
i_persp          <- ia.scale(ia.readAnimage(file.path(tiffdir, "persp_gretag_large.tif")), maxValue = 255)
i_perspDetail    <- list(ia.get(i_persp[[1]], animask(0,0,1201,256)),
                         ia.get(i_persp[[2]], animask(0,0,1201,256)),
                         ia.get(i_persp[[3]], animask(0,0,1201,256)))
#
plot(pixmapRGB(abind(i_perspDetail[[1]]@data,
                     i_perspDetail[[2]]@data,
                     i_perspDetail[[3]]@data,
                     rev.along=0)))
#
i_perspDetail_bayerized <- tm.bayerizeImage(i_perspDetail)
#
plot(pixmapGrey(i_perspDetail_bayerized@data),
     main="Bayerized version of a full RGB image")
#
i_perspDetail_debayerized <- tm.debayerizeImage(i_perspDetail_bayerized)
#
plot(pixmapRGB(abind(i_perspDetail_debayerized[[1]]@data,
                     i_perspDetail_debayerized[[2]]@data,
                     i_perspDetail_debayerized[[3]]@data,
                     rev.along=0)))
#
plot(pixmapGrey(i_perspDetail_debayerized[[1]]@data - i_perspDetail[[1]]@data),
     main="Difference of the de-bayerized image from the original (full RGB) version")
#
# Now some examples based on depth maps
#
i_perspDepth               <- tm.getDepth(file.path(tiffdir,
                                                    "pp_perspLabel_iso12233_large.ppm"), depthScale = 40)
i_perspDepthDetail         <- ia.get(i_perspDepth,     animask(0,0,1201,256))
i_perspRedDetail           <- ia.scale(i_perspRedDetail, maxValue = 255)
#
plot(pixmapGrey(i_perspDepthDetail@data), main="Depth map from camera")
#
i_perspRedDetail_blurred   <- tm.focusBlur(list(i_perspRedDetail),
                                           i_perspDepthDetail,
                                           21.0,
                                           8,
                                           50.0,
                                           scale     = 1201 / 36, # 1201 pixels over a 36 mm wide sensor
                                           maxRadius = 30.0)      # already quite blurred
#
plot(pixmapGrey(i_perspRedDetail_blurred[[1]]@data), main="Focus blur post processing for a 21mm lens")
#
# Now we want to experiment with log polar mapping
# we get a new smaller image detail to save on computation time
#
i_perspRedDetail <- ia.get(i_perspRed, animask(472,272,257,257))
#
# smooth it for an angular resolution of 72 pixels
#
i_perspRedDetail_smoothed <- tm.logPolarSmoothing(i_perspRedDetail, angularResolution = 72)
#
# and project/back project to get a feeling of the log polar mapping with
# 3960 pixels instead of 51471
#
i_perspRedDetail_lpe      <- tm.logPolarMapEquivalent(i_perspRedDetail_smoothed, 128, 72)
#
# and now crate the 'real' log polar map where the horizontal
# coordinate corresponds to rings, while the vertical one
# represents the angular 'sector'
#
i_perspRedDetail_lpm      <- tm.logPolarMap(i_perspRedDetail_smoothed, 128, angularResolution = 72)
#
# and we plot it with a gamma correction of 1.8
#
plot(pixmapGrey((i_perspRedDetail_lpm)**(1.0/1.8)),
     main="Log polar map with k=72")
#
# comparing it to the cartesian back projection (with a gamma correction of 1.8)
#
plot(pixmapGrey((i_perspRedDetail_lpe@data)**(1.0/1.8)),
     main="Cartesian equivalent of a log polar map with k=72")
#
# restore options
#
par(opar)
