\name{Anisotropic Diffusion}

\alias{Anisotropic Diffusion}
\alias{tm.anisotropicDiffusion}
\alias{tm.spaceVariantDiffusion}
\alias{tm.sampleGradientMap}

\title{Smoothing based on the heat equation}

\description{

  Images can be interpreted as heat maps that may be evolved in time
  according to the heat diffusion equation. The result is a blurred
  image whose degree of blurring is related to time.
}

\usage{
tm.sampleGradientMap(img)

tm.anisotropicDiffusion(img,  smap, maxSigma = 20, fh = 0.25)
tm.spaceVariantDiffusion(img, smap, maxSigma = 20, fh = 0.25)
}

\arguments{
  \item{img}{an \code{animage} (with values in the range $[0,1]$ except
	for \code{tm.sampleGradientMap})}
  \item{smap}{an array with the required smoothing for each pixel}
  \item{maxSigma}{a real representing the maximum smoothing ($\sigma$) required}
  \item{fh}{a real ($\leq 0.25$) controlling the update in the heat equation}
}

\value{
  The function \code{tm.sampleGradientMap} returns a real array. The
  other functions return a real animage.
}

\details{
  
  The functions  rely on the  heat diffusion equation to  obtain space
  variant  smoothing. The  function  \code{tm.anisotropicDiffusion} is
  based  on  a  modification  of the  original  anisotropic  diffusion
  approach (see References).   The modified version prevents intensity
  leakage and  can be  used to implement  depth of focus  effects. The
  values     of      map     \code{smap}     in      the     case     of
  \code{tm.anisotropicDiffusion} must belong to the interval [0,1]:
  pixels with a zero value do not contribute to smoothing. The
  function  \code{tm.spaceVariantDiffusion} is  based on  the original
  heat diffusion: the image is progressively smoothed and whenever the
  equivalent smoothing  $\sigma(t)$ exceeds the  value in \code{smap}
  (which in this case is not required to have values in the interval [0,1]),
  the corresponding pixel is stored in the final result image.

}

\author{Roberto Brunelli \email{brunelli@fbk.eu}}

\seealso{
  \link{Log Polar Sensors}
}

\references{
  Bertalmio, M., Fort, P., and Sanchez-Crespo D. (2004).
  \emph{Real-time, Accurate Depth of Field using Anisotropic Diffusion and
  Programmable Graphics Cards},  Proc. of 3DPVT'04, 767-773
}

\examples{
#
# Read sample image
#
tiffdir <-
  file.path(system.file(package = "TeMa"), "sampleimages/cameraSimulation")
i_persp <-
  ia.scale(ia.readAnimage(file.path(tiffdir, "persp_iso12233_large.tif")),
           maxValue = 1)
#
# extract a region
#
i_perspRedDetail <- ia.get(i_persp[[1]], animask(0,0,1201,128))
#
# generate the anisotropic map for the diffusion
#
gmap <- tm.sampleGradientMap(i_perspRedDetail)
#
# and perform anisotropic diffusion
#
i_perspRedDetail_smoothed <-
  tm.anisotropicDiffusion(i_perspRedDetail, gmap, maxSigma = 3)
}

\keyword{array}



