\name{illuminant-class}
\docType{class}
\alias{illuminant-class}
\alias{[,illuminant-method}
\alias{illuminant}
\alias{cie_d50}
\alias{cie_d65}

\title{Class "illuminant"}

\description{A simple structure to represent the spectral
  characteristics of CIE illuminants}

\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("illuminant", ...)}.

The resulting  objects can  be queried for  all the  intermediate values
using  the accessor  method.  


}

\section{Slots}{
	 \describe{
       \item{\code{label}:}{a string with the standard identification of
		 the illuminant.}
	   \item{\code{description}:}{A short description of the illuminant}
	   \item{\code{interpolator}:}{The interpolating function that
		 describes the spectral characteristics of the illuminant.}
  }
}

\usage{
illuminant(label, description, interpolator)
}

\arguments{
  
  \item{label}{a string providing the standard name of the illuminant}
  \item{description}{a string providing a short description of the illuminant}
  \item{interpolator}{an interpolating object (such as a
	\code{splinefun}) from which values can be extracted for any given
	(within range) argument}
  
}

\details{
  
  Two  commonly  used   illuminants  are  defined:  \code{cie_d50},  the
standard CIE daylight with a  correlated color temperature of 5000K, and
\code{cie_d65},  the  standard  CIE  daylight with  a  correlated  color
temperature of 6500K.

}

\examples{
cie_d50 <- illuminant()
cie_d65 <- illuminant(
               label = "CIE-D65",
               description =
                 "CIE daylight with a correlated color temperature of 6500K",
               interpolator = splinefun(
                 c(380, 385, 390, 395, 400, 405, 410, 415, 420, 425,
                   430, 435, 440, 445, 450, 455, 460, 465, 470, 475,
                   480, 485, 490, 495, 500, 505, 510, 515, 520, 525,
                   530, 535, 540, 545, 550, 555, 560, 565, 570, 575,
                   580, 585, 590, 595, 600, 605, 610, 615, 620, 625,
                   630, 635, 640, 645, 650, 655, 660, 665, 670, 675,
                   680, 685, 690, 695, 700, 705, 710, 715, 720, 725,
                   730, 735, 740, 745, 750, 755, 760),
                 c(0.499755, 0.523118, 0.546482, 0.687015, 0.827549,
                   0.871204, 0.914860, 0.924589, 0.934318, 0.900570,
                   0.866823, 0.957736, 1.048650, 1.109360, 1.170080,
                   1.174100, 1.178120, 1.163360, 1.148610, 1.153920,
                   1.159230, 1.123670, 1.088110, 1.090820, 1.093540,
                   1.085780, 1.07802,  1.06296,  1.0479,   1.06239,
                   1.076890, 1.06047,  1.04405,  1.04225,  1.04046,
                   1.020230, 1.00,     0.981671, 0.963342, 0.96061,
                   0.957880, 0.922368, 0.88686,  0.893459, 0.900062,
                   0.898026, 0.895991, 0.886489, 0.876987, 0.854936,
                   0.832890, 0.834939, 0.836992, 0.81863,  0.800268,
                   0.801207, 0.802146, 0.812462, 0.822778, 0.80281,
                   0.782842, 0.74003,  0.697213, 0.706652, 0.716091,
                   0.72979,  0.74349,  0.679765, 0.61604,  0.657448,
                   0.698856, 0.724863, 0.75087,  0.693398, 0.635927,
                   0.550054, 0.464182)))

}

\keyword{utilities}
