#include <arrayTools/Array.h>
#include <vector>
#include <unistd.h>

#define MAX(x, y) (((x) > (y)) ? (x) : (y))
#define MIN(x, y) (((x) < (y)) ? (x) : (y))
#define ABS(x)    (((x) > 0)   ? (x) : (-(x)))

using namespace std;

void ApplyMaskAt(ia::array<int> &M,
		 int             x,
		 int             y,
		 ia::array<int> &I,
		 ia::array<int> &X,
		 ia::array<int> &Y)
{
  int x0, x1, y0, y1;
  int n  = M.rows();
  int md = MAX(I.rows(), I.cols());
  int vx = md, vy = md, vs = 2*md*md;

  // Get boundaries
  //
  I.stat(x0, x1, y0, y1);

  // If we are at an object point
  //
  if(I[y][x] > 0) 
    {
      // Scan all mask points 
      //
      for(int i = 0; i < n; i++)
	{
	  // extract offset info from mask
	  //
	  int dx = M[i][0];
	  int dy = M[i][1];
	  int xx = x+dx;
	  int yy = y+dy;
	
	  if((x0 <= xx) && (xx <= x1) && (y0 <= yy) && (yy <= y1))
	    {
	      // add to stored vector
	      //
	      int wx = X[y+dy][x+dx]+dx;
	      int wy = Y[y+dy][x+dx]+dy;

	      // and look for minimum length vector
	      //
	      if(wx*wx + wy*wy < vs)
		{
		  // update vector components 
		  //
		  vx = wx;
		  vy = wy;

		  // and size
		  //
		  vs = wx*wx + wy*wy;
		}
	    }
	}

      // update distance maps
      //
      X[y][x] = vx;
      Y[y][x] = vy;
    }
}
	       
// Compute the signed Euclidean Transform of a given integer image
//
void SignedEuclideanDistanceTransform(ia::array<int>   &I,
				      ia::array<int>   &X,
				      ia::array<int>   &Y,
				      ia::array<double> &D)
{
  ia::array<int> Mask1(5,2);
  ia::array<int> Mask2(2,2);
  ia::array<int> Mask3(5,2);
  ia::array<int> Mask4(2,2);
  int            x0, x1, y0, y1;

  // Mask1 offsets
  //
  Mask1[0][0] =  0;
  Mask1[0][1] =  0;

  Mask1[1][0] = -1;
  Mask1[1][1] =  0;

  Mask1[2][0] = -1;
  Mask1[2][1] =  1;

  Mask1[3][0] =  0;
  Mask1[3][1] =  1;

  Mask1[4][0] =  1;
  Mask1[4][1] =  1;
  
  // Mask2 offsets
  //
  Mask2[0][0] =  0;
  Mask2[0][1] =  0;

  Mask2[1][0] =  1;
  Mask2[1][1] =  0;
  
  // Mask3 offsets
  //
  Mask3[0][0] =  0;
  Mask3[0][1] =  0;

  Mask3[1][0] =  1;
  Mask3[1][1] =  0;

  Mask3[2][0] =  1;
  Mask3[2][1] = -1;

  Mask3[3][0] =  0;
  Mask3[3][1] = -1;

  Mask3[4][0] = -1;
  Mask3[4][1] = -1;

  // Mask4 offsets
  //
  Mask4[0][0] =  0;
  Mask4[0][1] =  0;

  Mask4[1][0] = -1;
  Mask4[1][1] =  0;

  // Get image span
  //
  I.stat(x0, x1, y0, y1);

  // Initialize distance maps
  //
  int md = MAX(I.rows(), I.cols());

  X  = (*(new ia::array<int>   (y0, y1, x0, x1))).temporary();
  Y  = (*(new ia::array<int>   (y0, y1, x0, x1))).temporary();
  D  = (*(new ia::array<double> (y0, y1, x0, x1))).temporary();

  // Prepare pointers
  //
  int **_X     = X.storage();
  int **_Y     = Y.storage();
  int **_I     = I.storage();

  for(int y = y0; y <= y1; y++)
    for(int x = x0; x <= x1; x++)
      {
	// If we are on an object (i.e. >0) pixel
	//
	if(_I[y][x] > 0) 
	  {
	    _X[y][x] = _Y[y][x] = md;
	  }
	else 
	  {
	    // we are on a background (==0) pixel
	    // defining the zero-distance set
	    //
	    _X[y][x] = _Y[y][x] = 0;
	  }
      }

  // First round:  apply masks 1 and 2
  // in a bottom up manner
  //
  for(int y = y1; y >= y0; y--)
    {
      // Mask1 must be applied from left to right
      //
      for(int x = x0; x <= x1; x++)
	ApplyMaskAt(Mask1, x, y, I, X, Y);

      // while mask2 must be applied from right to left
      //
      for(int x = x1; x >= x0; x--)
	ApplyMaskAt(Mask2, x, y, I, X, Y);
    }
    
  // Second round: apply masks 3 and 4
  // in a top down manner
  //
  for(int y = y0; y <= y1; y++)
    {
      // Mask3 must be applied from right to left
      //
      for(int x = x1; x >= x0; x--)
      ApplyMaskAt(Mask3, x, y, I, X, Y);

      // while Mask4 from left to right 
      //
      for(int x = x0; x <= x1; x++)
	ApplyMaskAt(Mask4, x, y, I, X, Y);
    }

  // Fill indistance values
  //
  for(int y = y0; y <= y1; y++)
	for(int x = x0; x <= x1; x++)
	  D[y][x] = sqrt(double(_X[y][x]*_X[y][x]+_Y[y][x]*_Y[y][x]));
}

void Curvature(ia::array<u_char> &I, ia::array<int> &C)
{
  int x0, x1, y0, y1;

  I.stat(x0, x1, y0, y1);

  ia::array<int>   B(y0, y1, x0, x1);
  ia::array<int>   X(y0, y1, x0, x1);
  ia::array<int>   Y(y0, y1, x0, x1);
  ia::array<double> D(y0, y1, x0, x1);

  // Extract boundary
  //
  for(int y = y0+1; y < y1; y++)
    for(int x = x0+1; x < x1; x++)
      {
	int b = 1;

	if(I[y][x] > 0) 
	  {
	    for(int j = -1; (j <= 1) && b; j++)
	      for(int i = -1; (i <= 1) && b; i++)
		b = I[y+j][x+i];

	    B[y][x] = (b == 0) ? 255 : 0;
	  }
      }

  // Compute distance transform from boundary
  // considered as zero distance set
  //
  SignedEuclideanDistanceTransform(B, X, Y, D);

  // and compute curvature information
  //
  C  = (*(new ia::array<int> (y0, y1, x0, x1))).temporary();
  for(int y = y0; y <= y1; y++)
    for(int x = x0; x <= x1; x++)
      C[y+Y[y][x]][x+X[y][x]] += 1;
}
