#include <arrayTools/Array.h>

#define NABS(a)  	(((a) < (0)) ? (-(a)) : (a))
#define	EPS		1e-6
#define SIGN(x)		(((x) > 0.0) ? 1 : ((x) < 0.0) ? -1 : 0)

namespace _ARRAYTOOLS_NAMESPACE
{

  using std::log; 

char		mode[4];

//! One-Step Optimal V-Robust redescending operator for location from Hampel

double	TanhEstimatorPosition(double x, int derivative)
 {
   static int		iff = TRUE;
   static double	p = 1.634416, k = 4.5, r = 4.00, A = 0.804598, B = 0.877210; 
   static double	a, b;
   double		value, t;
  
   if(iff == TRUE)
      {
	iff 	= FALSE;
	a 	= sqrt(A*(k-1));
	b 	= 0.5*sqrt((k-1)*B*B/A);
      }
  
   if(NABS(x) >= r)
      {
	value = 0.0;
      }
   else
      {
	if(derivative == 0)
	   {
	     if(NABS(x) <= p)
		{
		  value = x;
		}
	     else
		{
		  value = a * tanh(b * (r - NABS(x))) * SIGN(x);
		}
	   }
	else
	   {
	     if(NABS(x) <= p)
		{
		  value = 1.0;
		}
	     else
		{
		  t	= 1.0 / cosh(b * (r - NABS(x)));
		 
		  value = -b * a * t * t * SIGN(x);
		}
	   }
      }
  
   return(value);
 }


//! One-Step Optimal V-Robust redescending operator for scale, from Hampel

double	TanhEstimatorScale(double x, int derivative)
 {
   static int		iff = TRUE;
   static double	p = 1.405979, aa = 0.293372, k = 6.0, r = 4.00;
   static double	A = 0.453990, B = 0.709875;
   static double	a, b;
   double		value, t;
  
   if(iff == TRUE)
      {
	iff 	= FALSE;
	a 	= sqrt(A*(k-1));
	b 	= 0.5*sqrt((k-1)*B*B/A);
      }
  
   if(NABS(x) >= r)
      {
	value = 0.0;
      }
   else
      {
	if(derivative == 0)
	   {
	     if(NABS(x) <= p)
		{
		  value = x*x - 1 + aa;
		}
	     else
		{
		  value = a * tanh(b * (log(r) - log(NABS(x))));
		}
	   }
	else
	   {
	     if(NABS(x) <= p)
		{
		  value = 2*x;
		}
	     else
		{
		  t	= 1.0 / cosh(b * (log(r) - log(NABS(x))));
		 
		  value = -b * a * t * t / x ;
		}
	   }
      }
  
   return(value);
 }


//! Function used by qsort.

static int CompareDouble(const void  *a, const void *b)
 {
   double	*A, *B;

   A = (double *) a;
   B = (double *) b;

   return((*A < *B) ? 1 : (*A > *B) ? -1 :  0);
 }
    

//!
void PositionAndScale(vector<double> &data,
		      char           mode,
		      double         *pos_return,
		      double         *scale_return,
		      int            c_pos,
		      int            c_scale)
 {
   double	ax, sx, dev;
   double	*buffer;
   int		i, n2, n2p;
   int          n = data.length();

   buffer = new double[n];
   
   switch(mode)
      {
       case	'p':
	 {
	   if(c_pos == 1)
	      {
		ax = 0.0;
		for(i = 0; i < n; i++)
		  ax	+= data[i];
		ax	/= n;
	      }
	   else
	     ax = *pos_return;
	
	   if(c_scale == 1)
	      {
		sx	= 0.0;
		for(i = 0; i < n; i++)
		  sx	+= data[i]*data[i];
		sx	= sqrt(sx/n - ax*ax);
	      }
	   else
	     sx	= *scale_return;
	
	   *pos_return	= ax;
	   *scale_return	= sx;
	 }
	break;
       case 	'm':
	  {
	    if(c_pos == 1)
	       {
		 ax	= 0.0;
		 for(i = 0; i < n; i++)
		   ax	+= data[i];
		 ax	/=  n;
	       }
	    else
	      ax	= *pos_return;
	
	    if(c_scale == 1)
	       {
		 sx	= 0.0;
		 for(i = 0; i < n; i++)
		   sx	+= NABS(data[i] - ax);
		 sx	/= n*sqrt(2.0/M_PI);
	       }
	    else
	      sx	= *scale_return;
	
	    *pos_return	= ax;
	    *scale_return	= sx;
	  }
	 break;
       case	'M':
       case	'O':
	  {
	    for(i = 0; i < n; i++)
	      buffer[i] = data[i];

	    n2p	= (n2=n/2) + 1;

	    if(c_pos == 1)
	       {
		 qsort((void *) buffer,
		       n,
		       sizeof(double),
		       CompareDouble);

		 ax	= (n%2) ? buffer[n2p] : 0.5 * (buffer[n2p] + buffer[n2]);
	       }
	    else
	      ax = *pos_return;
	
	    if(c_scale == 1)
	       {
		 dev = 0.0;
		 for(i = 0; i < n; i++)
		    {
		      buffer[i] 	= NABS(data[i] - ax);
		      dev 	+= buffer[i];
		    }
	
		 dev /= n;
	
		 qsort((char *)buffer,
		       n,
		       sizeof(double),
		       CompareDouble);

		 sx	= (n%2) ? buffer[n2p] : 0.5 * (buffer[n2p] + buffer[n2]);
		 sx	/= 0.6745;

		 if(sx == 0.0)
		    {
		      sx = dev/sqrt(2.0/M_PI);
		    }
	       }
	    else
	      sx	= *scale_return;
	
	    *pos_return	   = ax;
	    *scale_return  = sx;

	    if((mode == 'O') && (sx != 0.0))
	       {
		 double	loc 	= ax;
		 double	scale 	= sx;
		 double	a, b;
		 double	e_loc = 1.0, e_scale = 1.0;
		 int		kk;
	   
		 for(kk = 0; (kk < 1) && (e_loc > 0.001) && (e_scale > 0.001); kk++)
		    {
		
		      if(c_pos == 1)
			 {
			   a = 0.0;
			   b = 0.0;
	   
			   for(i = 0; i < n; i++)
			      {
				a += TanhEstimatorPosition((data[i] - ax)/sx, 0);
				b += TanhEstimatorPosition((data[i] - ax)/sx, 1);
			      }

			   if((a != 0.0) && (b != 0.0))
			     loc 		= ax + sx*(a/b);
			   else
			     loc		= ax;
		
			   *pos_return	= loc;
			   e_loc		= NABS((ax-loc)/loc);
			   ax			= loc;
			 }
		
		      if(c_scale == 1)
			 {
			   a = 0.0;
			   b = 0.0;
	   
			   for(i = 0; i < n; i++)
			      {
				a += TanhEstimatorScale((data[i] - loc)/sx, 0);
				b += (data[i]-loc)*TanhEstimatorScale((data[i]-loc)/sx,1);
			      }
	   
			   if((a != 0) && (b != 0.0))
			     scale 		= sx + sx*sx*(a/b);
			   else
			     scale		= sx;
		
			   *scale_return	= scale;
			   e_scale		= NABS((sx-scale)/sx);
			   sx  		= scale;
			 }
		    }
	       }
	  }
	break;
       default:
	fprintf(stderr, "Illegal option for PositionAndScale!\n");
	exit(-1);
      }

   delete buffer;
 }
    

void PositionAndScale(array<double>  &data,
		      char           mode,
		      vector<double> &pos_return,
		      vector<double> &scale_return,
		      int            c_pos,
		      int            c_scale)
 {
   int            d = data.cols();
   int            n = data.rows();
   vector<double> buffer(n);
   
   for(int i = 0; i < d; i++)
      {
	for(int j = 0; j < n; j++)
	  buffer[j] = data[j][i];
	
	PositionAndScale(buffer, 
			 mode, 
			 &(pos_return[i]),
			 &(scale_return[i]),
			 c_pos,
			 c_scale);
      }
 }

}
