#
# \litcode{Basic template matching}
#
# This function illustrates the most basic template matching technique:
# the template is moved over each image position and the sum of the squared
# difference of aligned image and template pixels is considered an indicator
# of template dissimilarity.
#
tm.basicTemplateMatching <- function(image, template) {
  #
  # The first step is to get the linear dimensions
  # of the template
  #
  dx <- ia.mask(template)@w
  dy <- ia.mask(template)@h
  #
  # We then determine the extent of the image region
  # of interest over which we need to slide our template
  #
  x0 <- ia.xmin(image)
  y0 <- ia.ymin(image) 
  x1 <- ia.xmax(image) - dx
  y1 <- ia.ymax(image) - dy
  #
  # This allows us to  prepare the storage for the similarity scores
  #
  scores <- animage(array(0,dim=c(y1-y0+1,x1-x0+1)),storage="real",focus=image@focus)
  #
  # and to  perform the actual computation loop,
  #
  for(y in y0:y1) {
    for(x in x0:x1) {
      #
      # sliding along each row
      # and extracting in turn an image portion
      # matching the size of the template
      #
      w <- ia.get(image, animask(x, y, dx, dy))
      #
      # We modify the template so that it overlaps
      # the extracted region
      #
      template@focus <- w@focus
      #
      # and compute (and store) the matching score
      #
      scores[y,x] <- sum(ia.matop(function(x,y) (x-y)**2, w, template)@data)
    }
  }
  #
  # We can finally return the image representing the similarity
  # map
  #
  scores
}
