require(Cairo)

tm.eps.on <- function(file, width = 4.5, height = 3.5, mar = c(4,4,1,0)) {
  postscript(file       = file,
             width      = width,
             height     = height,
             pointsize  = 9,
             horizontal = FALSE,
             onefile    = FALSE,
             paper      = "special",
             colormodel = "gray")
  
  par(mar=mar)
}

tm.eps.off <- function() {
  dev.off()
}

tm.show.ps <- function(image, file, ...) {
  tm.eps.on(file, ...)
  ia.show(image)
  tm.eps.off()
}

tm.ploteps <- function(file,
                       plotFunction,
                       addPlotFunction = NULL,
                       width = 4.5, height = 4.5, mar = c(4,4,1.5,0.5)) {

  
  if(exists("tm.override.eps", mode = "logical") &&
     (tm.override.eps == TRUE)) {
    eval(plotFunction)
    if(!is.null(addPlotFunction))
      eval(addPlotFunction)
  } else {
    postscript(file     = file,
               width      = width,
               height     = height,
               pointsize  = 9,
               horizontal = FALSE,
               onefile    = FALSE,
               paper      = "special",
               colormodel = "rgb")
    par(mar=mar, col=c("black"))
    
    eval(plotFunction)

    if(!is.null(addPlotFunction))
      eval(addPlotFunction)
    
    dev.off()
  }
}

tm.dev <- function(file = "",
                   width = 4.5, height = 4.5, mar = c(4,4,1.5,0.5),
                   format = NULL) {
  if(is.null(format)) {
    if(exists("tm.plot.defaultFormat", mode = "string")) {
      format <- tm.plot.defaultFormat
    } else {
      format <- "jpg"
    }
  }

  if(format == "eps") {
    postscript(file     = paste(file, "eps", sep="."),
               width      = width,
               height     = height,
               pointsize  = 9,
               horizontal = FALSE,
               onefile    = FALSE,
               paper      = "special",
               colormodel = "rgb")
    par(mar=mar)
  } else if (format == "jpg") {
    CairoJPEG(filename = paste(file, "jpg", sep="."),
              width = width * 200,
              height = height * 200,
              pointsize = 20);
  } else {
  }
}

tm.plot <- function(file,
                    plotFunction,
                    addPlotFunction = NULL,
                    width = 4.5, height = 4.5, mar = c(4,4,1.5,0.5),
                    format = NULL) {
  
  if(is.null(format)) {
    if(exists("tm.plot.defaultFormat", mode = "string")) {
      format <- tm.plot.defaultFormat
    } else {
      format <- "jpg"
    }
  }

  if(format == "eps") {
    postscript(file     = paste(file, "eps", sep="."),
               width      = width,
               height     = height,
               pointsize  = 9,
               horizontal = FALSE,
               onefile    = FALSE,
               paper      = "special",
               colormodel = "rgb")
    par(mar=mar)
  } else if (format == "jpg") {
    CairoJPEG(filename = paste(file, "jpg", sep="."),
              width = width * 200,
              height = height * 200,
              pointsize = 20);
  } else {
  }

  eval(plotFunction)
  
  if(!is.null(addPlotFunction))
    eval(addPlotFunction)
  
  dev.off()  
}



#
# From Prof. Ripley, a wrapper
#
## ploteps <- function(file, plotFunction, ...)
## {
##    Call <- match.call()
##    fn <- deparse(substitute(plotFunction))
##    Call[[1]] <- as.name(fn)
##    Call$file <- Call$plotFunction <- NULL
##    postscript(file=file)
##    eval.parent(Call)
##    dev.off()
## }
