#!/usr/bin/perl

#
# collate TemplateMatching.tex
#
# print to STDIN a single file obtained by processing all of the 
# input commands.
#
use FileHandle;

sub isCommentLine {
	my ($line) = @_;

	$line =~ s/^[\ \t]*[\#]+/\#/;
	if(substr($line, 0, 1) eq '#') {
		return(1);
	} else {
		return(0);
	}
}


sub ParseCode {
    my ($filename) = @_;
    my ($line);
    my $fh = new FileHandle; 
	my $codeLine    =  0;
	my $commentMode = -1;
	my $codeletMode = -1;
	my $codeMode    =  0;
	my $name        = $filename;

    open ($fh, $filename) || return; 

	$name =~ s/^[\.\/\w]+\/([\.\/\w]*)\.[\w]*/$1/g;

    while ($line = <$fh>) {
		if(isCommentLine($line) == 1) {
			$commentMode = 1;

			if($codeMode == 1) {
				print "\\end{lstlisting}\n";
				$codeMode    = 0;
			}

			$line =~ s/^[\ \t]*[\#]+[\ \t]*//;
			if(length($line) > 1) {
				#
				# we must check for codelet name
				# litcode
				#
				if($line =~ /\\litcode{([\w\s]+)}(.*)/) {
					$codeletMode = 1;
					print "\n\\begin{codelet}{$1}{$filename}$2\n";
				}

				print $line;
			}

		} else {
			$codeMode  = 1;
			$codeLine += 1;

			if($commentMode == 1) {
				if($codeLine == 1) {
					print "\\begin{lstlisting}[firstnumber=$codeLine,label=$name]\n";
				} else {
					print "\\begin{lstlisting}[firstnumber=$codeLine]\n";
				}
				$commentMode = 0;
			}
			print $line;
		}
	}

	if($codeMode == 1) {
		print "\\end{lstlisting}\n";
		$codeMode    = 0;
	}

	if($codeletMode == 1) {
		print "\\end{codelet}\n";
		$codeletMode    = 0;
	}

	close($fh)
}

sub ProcessFile {
    my ($filename) = @_;
    my ($line);
    my $fh;
	my $useSTDIN = 0;

	if(length($filename) == 0) {
		$useSTDIN = 1;
	}

	if($useSTDIN) {
		$fh = STDIN;
	} else {
		$fh = new FileHandle; 
		open ($fh, $filename) || return; 
	}

    while ($line = <$fh>) {
		if($line =~ /^\\\inputcode{([\w\.\/]*)}/) {
			$name = $1;

			print STDERR " - including $name ...\n";
			ParseCode($name);
		} elsif ($line =~ /^\\begin{Sinput}[\.]*/) {
			print "\\begin{lstlisting}[style=Sinput]\n";
		} elsif ($line =~ /^\\begin{Soutput}[\.]*/) {
			print "\\begin{lstlisting}[style=Soutput]\n";
		} elsif ($line =~ /^\\end{Sinput}[\.]*/) {
			print "\\end{lstlisting}\n";
		} elsif ($line =~ /^\\end{Soutput}[\s]*/) {
			print "\\end{lstlisting}\n";
		} elsif ($line =~ /\\litcode{([\w\s]+)}(.+)/) {
			print "\\begin{codelet}[$1]\n$2\n";
		} else {
			print $line;
		}
    }

	if($useSTDIN) {
		;
	} else {
		close($fh);
	}
}

print STDERR "Expanding LaTeX inputcode directives ...\n";

$main = shift(@ARGV);

$a    = &ProcessFile($main);


