\name{ia.gaussianSmooth}

\alias{ia.gaussianSmooth}
\alias{ia.zeroCrossings}

\title{Gaussian smoothing}

\description{

   Smooth an  integer (real) \code{animage} with a Gaussian kernel

}

\usage{

ia.gaussianSmooth(img, sigma = 1.0)
ia.zeroCrossings(img,  sigma = 1.0)

}

\arguments{
   	\item{img}{an integer (real) \code{animage}}
	\item{sigma}{a real representing the standard deviation of the
	  Gaussian kernel}
}

\value{

 Function \code{ia.gaussianSmooth} returns an  \code{animage} of the same type as
 \code{img} while function \code{ia.zeroCrossings} returns an integer
 \code{animage}.
 
}

\details{
  
  Function \code{ia.gaussianSmooth} convolves the given \code{animage}
  with the specified Gaussian kernel exploiting its separability for
  better performance. Smoothing is limited to the
  animage region of interest. Function \code{ia.zeroCrossings} computes
  the zero crossings the laplacian image approximated with the
  difference of two images smoothed with Gaussian kernels whose sigmas
  have a 1.6 ratio.

}

\author{Roberto Brunelli \email{brunelli@fbk.eu}}
\references{
  D. Marr, \emph{Vision}, W.H. Freeman, 1982
}

\examples{
#
# get paths to package samples
#
sampleimages <- file.path(system.file(package = "AnImAl"), "sampleimages")
#
# Read a sample images
#
charA <- ia.readAnimageChannel(file.path(sampleimages, "a.tif"), 1)
#
# set the animage outside to white so that we do not get
# spurious effects at the boundary 
#
smoothedCharA <- ia.gaussianSmooth(charA, sigma = 3.0)
#
# have a look
#
ia.show(smoothedCharA)
#
# A simple scale space (4 scales) zero-crossings representation
#
sampleFace <- as.animage(getChannels(read.pnm(file.path(sampleimages,
"sampleFace_01.pgm"))))
#
oldpar        <- par(no.readonly = TRUE)
#
par(mfrow   = c(2,2),
    mar     = c(3,0,0,0),
    mgp     = c(1,1,1),
    oma     = c(0,0,3,0),
    cex.lab = 1.5)
#
for(sigma in c(0.5, 1, 2, 4)) {
 ia.show(ia.not(ia.zeroCrossings(sampleFace, sigma)),
         xlab=paste("Sigma = ", sigma))
}
#
mtext("Zero-crossings scale space", outer = TRUE, side = 3, cex=1.5,
line=1)
par(oldpar)
#
}
\keyword{array}
